@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -90,8 +88,12 @@ byte_swap_sbin(struct fs *sb)
 	byte_swap_ints(((int32_t *)&sb->fs_firstfield), 52);
 	byte_swap_int(sb->fs_cgrotor);
 	byte_swap_int(sb->fs_cpc);
-	byte_swap_shorts((int16_t *)sb->fs_opostbl, 16 * 8); 
-	byte_swap_ints((int32_t *)sb->fs_sparecon, 50);
+        byte_swap_shorts((int16_t *)sb->fs_opostbl,
+                         sizeof(sb->fs_opostbl) / sizeof(int16_t));
+        byte_swap_int(sb->fs_avgfilesize);
+        byte_swap_int(sb->fs_avgfpdir);
+        byte_swap_ints((int32_t *)sb->fs_sparecon,
+                       sizeof(sb->fs_sparecon) / sizeof(int32_t));
 	byte_swap_ints((int32_t *)&sb->fs_contigsumsize, 3);
 	byte_swap_longlongs((u_int64_t *)&sb->fs_maxfilesize,3);
 	byte_swap_ints((int32_t *)&sb->fs_state, 6);
@@ -119,8 +121,12 @@ byte_swap_sbout(struct fs *sb)
 	byte_swap_ints(((int32_t *)&sb->fs_firstfield), 52);
 	byte_swap_int(sb->fs_cgrotor);
 	byte_swap_int(sb->fs_cpc);
-	byte_swap_shorts((int16_t *)sb->fs_opostbl, 16 * 8); 
-	byte_swap_ints((int32_t *)sb->fs_sparecon, 50);
+        byte_swap_shorts((int16_t *)sb->fs_opostbl,
+                         sizeof(sb->fs_opostbl) / sizeof(int16_t));
+        byte_swap_int(sb->fs_avgfilesize);
+        byte_swap_int(sb->fs_avgfpdir);
+        byte_swap_ints((int32_t *)sb->fs_sparecon,
+                       sizeof(sb->fs_sparecon) / sizeof(int32_t));
 	byte_swap_ints((int32_t *)&sb->fs_contigsumsize, 3);
 	byte_swap_longlongs((u_int64_t *)&sb->fs_maxfilesize,3);
 	byte_swap_ints((int32_t *)&sb->fs_state, 6);
@@ -156,7 +162,7 @@ byte_swap_cgin(struct cg *cg, struct fs * fs)
 	byte_swap_int(cg->cg_nextfreeoff);
 	byte_swap_int(cg->cg_clusteroff);
 	byte_swap_int(cg->cg_nclusterblks);
-	byte_swap_ints(&cg->cg_sparecon, 13);
+	byte_swap_ints(cg->cg_sparecon, 13);
 
 	byte_swap_int(cg->cg_btotoff);
 	ulptr = ((int32_t *)((u_int8_t *)(cg) + (cg)->cg_btotoff));
@@ -202,7 +208,7 @@ byte_swap_cgout(struct cg *cg, struct fs * fs)
 	byte_swap_int(cg->cg_freeoff);
 	byte_swap_int(cg->cg_nextfreeoff);
 	byte_swap_int(cg->cg_nclusterblks);
-	byte_swap_ints(&cg->cg_sparecon, 13);
+	byte_swap_ints(cg->cg_sparecon, 13);
 
 	byte_swap_int(cg->cg_iusedoff);
 	byte_swap_int(cg->cg_clusteroff);
