@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -39,10 +37,11 @@
 
 #include <sys/types.h>
 #include <sys/file.h>
-#include <dev/disk.h>
+#include <sys/disk.h>
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
+#include <unistd.h>
 
 char *blockcheck __P((char *));
 
@@ -64,7 +63,7 @@ long dksecsize (dev)
     else
           return (0);
 
-    if (ioctl(fd, DKIOCBLKSIZE, &devblklen) < 0) {
+    if (ioctl(fd, DKIOCGETBLOCKSIZE, &devblklen) < 0) {
 	(void)close(fd);
         return (0);
     }
