@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -61,9 +59,9 @@
 #include <sys/ioctl.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+
 #ifdef sunos
 #include <sys/vnode.h>
-
 #include <ufs/inode.h>
 #else
 #include <ufs/ufs/dinode.h>
@@ -80,6 +78,8 @@
 #include <pwd.h>
 #include <signal.h>
 #include <stdio.h>
+#include <errno.h>
+
 #ifdef __STDC__
 #include <stdlib.h>
 #include <string.h>
@@ -229,7 +229,6 @@ rmtread(buf, count)
 {
 	char line[30];
 	int n, i, cc;
-	extern errno;
 
 	(void)sprintf(line, "R%d\n", count);
 	n = rmtcall("read", line);
