@@ -2,21 +2,22 @@
  * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -29,6 +30,7 @@
 #include <sys/types.h>
 #include <sys/uio.h>
 #include <unistd.h>
+#include <string.h>
 
 #include "cache.h"
 
@@ -745,6 +747,75 @@ CacheFlush( Cache_t *cache )
 } /* CacheFlush */
 
 
+/*
+ * RangeIntersect
+ *
+ * Return true if the two given ranges intersect.
+ *
+ */
+static int
+RangeIntersect(uint64_t start1, uint64_t len1, uint64_t start2, uint64_t len2)
+{
+	uint64_t end1 = start1 + len1 - 1;
+	uint64_t end2 = start2 + len2 - 1;
+	
+	if (end1 < start2 || start1 > end2)
+		return 0;
+	else
+		return 1;
+}
+
+
+/*
+ * CacheFlushRange
+ *
+ * Flush, and optionally remove, all cache blocks that intersect
+ * a given range.
+ */
+int
+CacheFlushRange( Cache_t *cache, uint64_t start, uint64_t len, int remove)
+{
+	int error;
+	int i;
+	Tag_t *currentTag, *nextTag;
+	
+	for ( i = 0; i < cache->HashSize; i++ )
+	{
+		currentTag = cache->Hash[ i ];
+		
+		while ( NULL != currentTag )
+		{
+			/* Keep track of the next block, in case we remove the current block */
+			nextTag = currentTag->Next;
+
+			if ( currentTag->Flags & kLazyWrite &&
+				 RangeIntersect(currentTag->Offset, cache->BlockSize, start, len))
+			{
+				error = CacheRawWrite( cache,
+									   currentTag->Offset,
+									   cache->BlockSize,
+									   currentTag->Buffer );
+				if ( EOK != error )
+				{
+#if CACHE_DEBUG
+					printf( "%s - CacheRawWrite failed with error %d \n", __FUNCTION__, error );
+#endif 
+					return error;
+				}
+				currentTag->Flags &= ~kLazyWrite;
+
+				if ( remove )
+					CacheRemove( cache, currentTag );
+			}
+			
+			currentTag = nextTag;
+		} /* while */
+	} /* for */
+	
+	return EOK;
+} /* CacheFlushRange */
+
+
 /*
  * CacheLookup
  *
