@@ -1,22 +1,23 @@
 /*
- * Copyright (c) 2000-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2000-2003, 2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -80,6 +81,7 @@ GetVolumeBlock (SVCB *volume, UInt64 blockNum, GetBlockOptions options, BlockDes
 	if (result == 0) {
 		block->blockHeader = buffer;
 		block->buffer = buffer->Buffer;
+		block->blockNum = blockNum;
 		block->blockSize = blockSize;
 		block->blockReadFromDisk = 0;
 		block->fragmented = 0;
@@ -179,6 +181,7 @@ GetFileBlock (SFCB *file, UInt32 blockNum, GetBlockOptions options, BlockDescrip
 
 	block->blockHeader = buffer;
 	block->buffer = buffer->Buffer;
+	block->blockNum = blockNum;
 	block->blockSize = file->fcbBlockSize;
 	block->blockReadFromDisk = 0;
 	block->fragmented = 0;
@@ -275,6 +278,7 @@ ReadFragmentedBlock (SFCB *file, UInt32 blockNum, BlockDescriptor *block)
 	
 	block->buffer = buffer;
 	block->blockHeader = bufs;
+	block->blockNum = blockNum;
 	block->blockSize = blockSize;
 	block->blockReadFromDisk = false;
 	block->fragmented = true;
