@@ -2,21 +2,22 @@
  * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -67,8 +68,8 @@ static int  CheckCatalogName(u_int16_t charCount, const u_int16_t *uniChars,
 static int  CheckCatalogName_HFS(u_int16_t charCount, const u_char *filename,
                                  u_int32_t parentID, Boolean thread);
 
-static int  RecordBadAllocation(UInt32 parID, char * filename, UInt32 forkType, UInt32 oldBlkCnt, UInt32 newBlkCnt);
-static int  RecordTruncation(UInt32 parID, char * filename, UInt32 forkType, UInt64 oldSize,  UInt64 newSize);
+static int  RecordBadAllocation(UInt32 parID, unsigned char * filename, UInt32 forkType, UInt32 oldBlkCnt, UInt32 newBlkCnt);
+static int  RecordTruncation(UInt32 parID, unsigned char * filename, UInt32 forkType, UInt64 oldSize,  UInt64 newSize);
 
 static int  CaptureMissingThread(UInt32 threadID, const HFSPlusCatalogKey *nextKey);
 static 	OSErr	UniqueDotName( 	SGlobPtr GPtr, 
@@ -78,6 +79,14 @@ static 	OSErr	UniqueDotName( 	SGlobPtr GPtr,
                                 Boolean isHFSPlus );
 static Boolean 	FixDecomps(	u_int16_t charCount, const u_int16_t *inFilename, HFSUniStr255 *outFilename );
 
+static int FindOrigOverlapFiles(SGlobPtr GPtr);
+static void CheckHFSPlusExtentRecords(SGlobPtr GPtr, UInt32 fileID, HFSPlusExtentRecord extent, UInt8 forkType); 
+static void CheckHFSExtentRecords(SGlobPtr GPtr, UInt32 fileID, HFSExtentRecord extent, UInt8 forkType);
+static Boolean DoesOverlap(SGlobPtr GPtr, UInt32 fileID, UInt32 startBlock, UInt32 blockCount, UInt8 forkType); 
+
+void PrintOverlapFiles (SGlobPtr GPtr);
+static int CompareExtentFileID(const void *first, const void *second);
+
 /*
  * CheckCatalogBTree - Verifies the catalog B-tree structure
  *
@@ -109,14 +118,32 @@ CheckCatalogBTree( SGlobPtr GPtr )
 	err = BTCheck(gScavGlobals, kCalculatedCatalogRefNum, (CheckLeafRecordProcPtr)CheckCatalogRecord);
 	if (err) goto exit;
 
-	if (gCIS.dirCount != gCIS.dirThreads)
+	if (GPtr->VIStat & S_OverlappingExtents) {
+		/* If BTCheck found any overlapped extents, fsck_hfs has reported 
+		 * all the fileIDs except the original fileIDs.  Traverse thei volume header,
+	 	 * catalog btree and extents btree to find the original fileIDs 
+	 	 */
+		err = FindOrigOverlapFiles(GPtr);
+		if (err) goto exit;
+		
+		/* Print all unique overlapping file IDs and paths */
+		(void) PrintOverlapFiles(GPtr);
+	}
+
+	if (gCIS.dirCount != gCIS.dirThreads) {
+		RcdError(gScavGlobals, E_IncorrectNumThdRcd);
 		gScavGlobals->CBTStat |= S_Orphan;  /* a directory record is missing */
+	}
 
-	if (hfsplus && (gCIS.fileCount != gCIS.fileThreads))
+	if (hfsplus && (gCIS.fileCount != gCIS.fileThreads)) {
+		RcdError(gScavGlobals, E_IncorrectNumThdRcd);
 		gScavGlobals->CBTStat |= S_Orphan;
+	}
 
-	if (!hfsplus && (gCIS.fileThreads != gCIS.filesWithThreads))
+	if (!hfsplus && (gCIS.fileThreads != gCIS.filesWithThreads)) {
+		RcdError(gScavGlobals, E_IncorrectNumThdRcd);
 		gScavGlobals->CBTStat |= S_Orphan;
+	}
 
 	gScavGlobals->calculatedVCB->vcbEncodingsBitmap = gCIS.encodings;
 	gScavGlobals->calculatedVCB->vcbNextCatalogID = gCIS.nextCNID;
@@ -323,16 +350,16 @@ static int
 CheckIfAttributeExists(const UInt32 fileID) 
 {
 	int result = 0;
-	AttributeKey *attrKey;
+	HFSPlusAttrKey *attrKey;
 	BTreeIterator iterator;
 
 	/* Initialize the iterator, note that other fields in iterator are initialized to zero */
 	ClearMemory(&iterator, sizeof(BTreeIterator));
 
 	/* Initialize attribute key to iterator key.  Other fields initialized to zero */
-	attrKey = (AttributeKey *)&iterator.key;
-	attrKey->keyLength = kAttributeKeyMinimumLength;
-	attrKey->cnid = fileID;
+	attrKey = (HFSPlusAttrKey *)&iterator.key;
+	attrKey->keyLength = kHFSPlusAttrKeyMinimumLength;
+	attrKey->fileID = fileID;
 
 	/* Search for attribute with NULL name.  This will place the iterator at correct fileID location in BTree */	
 	result = BTSearchRecord(gScavGlobals->calculatedAttributesFCB, &iterator, kInvalidMRUCacheKey, NULL, NULL, &iterator);
@@ -345,7 +372,7 @@ CheckIfAttributeExists(const UInt32 fileID)
 
 	/* Iterate to next record and check there is a valid extended attribute for this file ID */ 
 	result = BTIterateRecord(gScavGlobals->calculatedAttributesFCB, kBTreeNextRecord, &iterator, NULL, NULL);
-	if ((result == noErr) && (attrKey->cnid == fileID)) {
+	if ((result == noErr) && (attrKey->fileID == fileID)) {
 		/* Set success return value only if we did _find_ an attribute record for the current fileID */
 		result = 1;
 	} 
@@ -370,6 +397,7 @@ CheckDirectory(const HFSPlusCatalogKey * key, const HFSPlusCatalogFolder * dir)
 
 	dirID = dir->folderID;
 
+	/* Directory cannot have these two flags set */
 	if ((dir->flags & (kHFSFileLockedMask | kHFSThreadExistsMask)) != 0) {
 		RcdError(gScavGlobals, E_CatalogFlagsNotZero);
 		gScavGlobals->CBTStat |= S_ReservedNotZero;
@@ -381,7 +409,7 @@ CheckDirectory(const HFSPlusCatalogKey * key, const HFSPlusCatalogFolder * dir)
 	result = CheckIfAttributeExists(dir->folderID);
 	if (result == 1) {
 		/* Atleast one extended attribute exists for this folder ID */
-		/* 3857929: Record if directory record has attribute and/or security bit set */
+		/* 3843779 : Record if directory record has attribute and/or security bit set */
 		RecordXAttrBits(gScavGlobals, dir->flags, dir->folderID, kCalculatedCatalogRefNum);
 #if DEBUG_XATTR
 		printf ("%s: Record folderID=%d for prime modulus calculations\n", __FUNCTION__, dir->folderID); 
@@ -427,14 +455,14 @@ CheckFile(const HFSPlusCatalogKey * key, const HFSPlusCatalogFile * file)
 	UInt64 bytes;
 	int result = 0;
 	size_t	len;
-	char filename[256 * 3];
+	unsigned char filename[256 * 3];
 
 	(void) utf_encodestr(key->nodeName.unicode,
 				key->nodeName.length * 2,
 				filename, &len);
 	filename[len] = '\0';
 
-	/* 3843766: Check for reserved bits removed to support new bits in future */
+	/* 3843017 : Check for reserved field removed to support new bits in future */
 
 	/* 3971173, 3977448 : HFS does not clear the attribute bit after it deletes the 
 	 * last attribute.  Fix it later in 3977448 by removing the call for checking attrs
@@ -442,7 +470,7 @@ CheckFile(const HFSPlusCatalogKey * key, const HFSPlusCatalogFile * file)
 	result = CheckIfAttributeExists(file->fileID);
 	if (result == 1) {
 		/* Atleast one extended attribute exists for this file ID */
-		/* 3857929: Record if file record has attribute and/or security bit set */
+		/* 3843779 : Record if file record has attribute and/or security bit set */
 		RecordXAttrBits(gScavGlobals, file->flags, file->fileID, kCalculatedCatalogRefNum);
 #if DEBUG_XATTR
 		printf ("%s: Record fileID=%d for prime modulus calculations\n", __FUNCTION__, file->fileID); 
@@ -577,6 +605,7 @@ CheckDirectory_HFS(const HFSCatalogKey * key, const HFSCatalogFolder * dir)
 
 	dirID = dir->folderID;
 
+	/* Directory cannot have these two flags set */
 	if ((dir->flags & (kHFSFileLockedMask | kHFSThreadExistsMask)) != 0) {
 		RcdError(gScavGlobals, E_CatalogFlagsNotZero);
 		gScavGlobals->CBTStat |= S_ReservedNotZero;
@@ -613,7 +642,7 @@ CheckFile_HFS(const HFSCatalogKey * key, const HFSCatalogFile * file)
 	if (file->flags & kHFSThreadExistsMask)
 		++gCIS.filesWithThreads;
 
-	/* 3843766: Check for reserved bits removed to support new bits in future */
+	/* 3843017 : Check for reserved field removed to support new bits in future */
 	if ((file->dataStartBlock)          ||
 	    (file->rsrcStartBlock)          ||
 	    (file->reserved))
@@ -969,7 +998,7 @@ CheckCatalogName_HFS(u_int16_t charCount, const u_char *filename, u_int32_t pare
             if ( roPtr == NULL ) 
                 return( noErr );
  
-            myPtr = &roPtr->name[0];
+            myPtr = (u_char *)&roPtr->name[0];
             *myPtr++ = charCount; // copy in length of old name and bump past it
             CopyMemory( filename, myPtr, charCount );
             myPtr += charCount; // bump past old name
@@ -1020,7 +1049,7 @@ UniqueDotName( 	SGlobPtr GPtr,
     else
         myPtr = &dotName[0];
     
-	nameLen = strlen( myPtr );
+	nameLen = strlen((char *) myPtr );
     if ( isHFSPlus )
     {
         int		i;
@@ -1071,7 +1100,7 @@ UniqueDotName( 	SGlobPtr GPtr,
  * count isn't large enough to cover the current LEOF.
  */
 static int
-RecordBadAllocation(UInt32 parID, char * filename, UInt32 forkType, UInt32 oldBlkCnt, UInt32 newBlkCnt)
+RecordBadAllocation(UInt32 parID, unsigned char * filename, UInt32 forkType, UInt32 oldBlkCnt, UInt32 newBlkCnt)
 {
 	RepairOrderPtr 			p;
 	char 					goodstr[16];
@@ -1089,7 +1118,7 @@ RecordBadAllocation(UInt32 parID, char * filename, UInt32 forkType, UInt32 oldBl
 	if ( !isHFSPlus )
 		return (E_PEOF);
 
-	n = strlen(filename);
+	n = strlen((char *)filename);
 	p = AllocMinorRepairOrder(gScavGlobals, n + 1);
 	if (p == NULL)
 		return (R_NoMem);
@@ -1114,7 +1143,7 @@ RecordBadAllocation(UInt32 parID, char * filename, UInt32 forkType, UInt32 oldBl
  * Record a repair to trucate a file's logical size.
  */
 static int
-RecordTruncation(UInt32 parID, char * filename, UInt32 forkType, UInt64 oldSize, UInt64 newSize)
+RecordTruncation(UInt32 parID, unsigned char * filename, UInt32 forkType, UInt64 oldSize, UInt64 newSize)
 {
 	RepairOrderPtr	 		p;
 	char 					oldSizeStr[48];
@@ -1132,7 +1161,7 @@ RecordTruncation(UInt32 parID, char * filename, UInt32 forkType, UInt64 oldSize,
 	if ( !isHFSPlus )
 		return (E_LEOF);
 
-	n = strlen(filename);
+	n = strlen((char *)filename);
 	p = AllocMinorRepairOrder(gScavGlobals, n + 1);
 	if (p == NULL)
 		return (R_NoMem);
@@ -1467,5 +1496,369 @@ FixDecomps(	u_int16_t charCount, const u_int16_t *inFilename, HFSUniStr255 *outF
     
 } /* FixDecomps */
 
+/* Function :  DoesOverlap
+ * 
+ * Description: 
+ * This function takes a start block and the count of blocks in a 
+ * given extent and compares it against the list of overlapped 
+ * extents in the global structure.   
+ * This is useful in finding the original files that overlap with
+ * the files found in catalog btree check.  If a file is found
+ * overlapping, it is added to the overlap list. 
+ * 
+ * Input: 
+ * 1. GPtr - global scavenger pointer.
+ * 2. fileID - file ID being checked.
+ * 3. startBlock - start block in extent.
+ * 4. blockCount - total number of blocks in extent.
+ * 5. forkType - type of fork being check (kDataFork, kRsrcFork).
+ * 
+ * Output: isOverlapped - Boolean value of true or false.
+ */
+static Boolean DoesOverlap(SGlobPtr GPtr, UInt32 fileID, UInt32 startBlock, UInt32 blockCount, UInt8 forkType) 
+{
+	int i;
+	Boolean isOverlapped = false;
+	ExtentInfo	*curExtentInfo;
+	ExtentsTable **extentsTableH = GPtr->overlappedExtents;
 
+	for (i = 0; i < (**extentsTableH).count; i++) {
+		curExtentInfo = &((**extentsTableH).extentInfo[i]);
+		/* Check extents */
+		if (curExtentInfo->startBlock < startBlock) {
+			if ((curExtentInfo->startBlock + curExtentInfo->blockCount) > startBlock) {
+				isOverlapped = true;
+				break;
+			}
+		} else {	/* curExtentInfo->startBlock >= startBlock */
+			if (curExtentInfo->startBlock < (startBlock + blockCount)) {
+				isOverlapped = true;
+				break;
+			}
+		}
+	} /* for loop Extents Table */	
 
+	/* Add this extent to overlap list */
+	if (isOverlapped) {
+		AddExtentToOverlapList(GPtr, fileID, startBlock, blockCount, forkType);
+	}
+
+	return isOverlapped;
+} /* DoesOverlap */
+
+/* Function : CheckHFSPlusExtentRecords
+ * 
+ * Description: 
+ * For all valid extents, this function calls DoesOverlap to find
+ * if a given extent is overlapping with another extent existing
+ * in the overlap list.
+ * 
+ * Input: 
+ * 1. GPtr - global scavenger pointer.
+ * 2. fileID - file ID being checked.
+ * 3. extent - extent information to check.
+ * 4. forkType - type of fork being check (kDataFork, kRsrcFork).
+ * 
+ * Output: None.
+ */
+static void CheckHFSPlusExtentRecords(SGlobPtr GPtr, UInt32 fileID, HFSPlusExtentRecord extent, UInt8 forkType) 
+{
+	int i;
+
+	/* Check for overlapping extents for all extents in given extent data */
+	for (i = 0; i < kHFSPlusExtentDensity; i++) {
+		if (extent[i].startBlock == 0) {
+			break;
+		}
+		DoesOverlap(GPtr, fileID, extent[i].startBlock, extent[i].blockCount, forkType);
+	} 
+	return;
+} /* CheckHFSPlusExtentRecords */ 
+
+/* Function : CheckHFSExtentRecords
+ * 
+ * Description: 
+ * For all valid extents, this function calls DoesOverlap to find
+ * if a given extent is overlapping with another extent existing
+ * in the overlap list.
+ * 
+ * Input: 
+ * 1. GPtr - global scavenger pointer.
+ * 2. fileID - file ID being checked.
+ * 3. extent - extent information to check.
+ * 4. forkType - type of fork being check (kDataFork, kRsrcFork).
+ * 
+ * Output: None.
+ */
+static void CheckHFSExtentRecords(SGlobPtr GPtr, UInt32 fileID, HFSExtentRecord extent, UInt8 forkType) 
+{
+	int i;
+
+	/* Check for overlapping extents for all extents in given extents */
+	for (i = 0; i < kHFSExtentDensity; i++) {
+		if (extent[i].startBlock == 0) {
+			break;
+		}
+		DoesOverlap(GPtr, fileID, extent[i].startBlock, extent[i].blockCount, forkType);
+	}
+	return;
+} /* CheckHFSExtentRecords */ 
+
+/* Function: FindOrigOverlapFiles 
+ * 
+ * Description:
+ * This function is called only if catalog btree check results in
+ * overlapped extents errors.  The catalog btree check does not find
+ * out the original files whose extents are overlapping with one
+ * being reported in its check.  This function finds out all the 
+ * original files whose that are being overlapped.  
+ * 
+ * This function relies on comparison of extents with Overlap list
+ * created in verify stage.  The list is also updated with the 
+ * overlapped extents found in this function. 
+ * 
+ * 1. Compare extents for all the files located in volume header.
+ * 2. Traverse catalog btree and compare extents of all files.
+ * 3. Traverse extents btree and compare extents for all entries.
+ * 
+ * Input: GPtr - pointer to global scanvenger area.
+ * 
+ * Output: err - function result
+ *			zero means success
+ *			non-zero means failure
+ */
+static int FindOrigOverlapFiles(SGlobPtr GPtr)
+{
+	OSErr err = noErr;
+	Boolean isHFSPlus; 
+
+	UInt16 selCode;		/* select access pattern for BTree */
+	UInt16 recordSize;
+	UInt32 hint;
+	CatalogRecord catRecord; 
+	CatalogKey catKey;
+	ExtentRecord extentRecord; 
+	ExtentKey extentKey;
+	SVCB *calculatedVCB = GPtr->calculatedVCB;
+
+	isHFSPlus = VolumeObjectIsHFSPlus();
+
+	/* Check file extents from volume header */
+	if (isHFSPlus) {
+		/* allocation file */
+		if (calculatedVCB->vcbAllocationFile) {
+			CheckHFSPlusExtentRecords(GPtr, calculatedVCB->vcbAllocationFile->fcbFileID, 
+		                              calculatedVCB->vcbAllocationFile->fcbExtents32, kDataFork);
+		}
+
+		/* extents file */
+		if (calculatedVCB->vcbExtentsFile) {
+			CheckHFSPlusExtentRecords(GPtr, calculatedVCB->vcbExtentsFile->fcbFileID, 
+		                              calculatedVCB->vcbExtentsFile->fcbExtents32, kDataFork);
+		}
+
+		/* catalog file */
+		if (calculatedVCB->vcbCatalogFile) {
+			CheckHFSPlusExtentRecords(GPtr, calculatedVCB->vcbCatalogFile->fcbFileID, 
+		                              calculatedVCB->vcbCatalogFile->fcbExtents32, kDataFork);
+		}
+
+		/* attributes file */
+		if (calculatedVCB->vcbAttributesFile) {
+			CheckHFSPlusExtentRecords(GPtr, calculatedVCB->vcbAttributesFile->fcbFileID, 
+		                              calculatedVCB->vcbAttributesFile->fcbExtents32, kDataFork);	
+	   	}
+
+		/* startup file */
+		if (calculatedVCB->vcbStartupFile) {
+			CheckHFSPlusExtentRecords(GPtr, calculatedVCB->vcbStartupFile->fcbFileID, 
+		                              calculatedVCB->vcbStartupFile->fcbExtents32, kDataFork);
+		}
+	} else {
+		/* extents file */
+		if (calculatedVCB->vcbExtentsFile) {
+			CheckHFSExtentRecords(GPtr, calculatedVCB->vcbExtentsFile->fcbFileID, 
+		                          calculatedVCB->vcbExtentsFile->fcbExtents16, kDataFork);
+		}
+
+		/* catalog file */
+		if (calculatedVCB->vcbCatalogFile) {
+			CheckHFSExtentRecords(GPtr, calculatedVCB->vcbCatalogFile->fcbFileID, 
+		                          calculatedVCB->vcbCatalogFile->fcbExtents16, kDataFork);
+		}
+	}
+
+	/* Traverse the catalog btree */ 
+	selCode = 0x8001;	/* Get first record from BTree */
+	err = GetBTreeRecord(GPtr->calculatedCatalogFCB, selCode, &catKey, &catRecord, &recordSize, &hint);
+	if (err != noErr) {
+		goto traverseExtents;
+	} 
+	selCode = 1;	/* Get next record */
+	do {
+		if ((catRecord.recordType == kHFSPlusFileRecord) ||  
+		    (catRecord.recordType == kHFSFileRecord)) {
+			
+			if (isHFSPlus) {
+				/* HFSPlus data fork */
+				CheckHFSPlusExtentRecords(GPtr, catRecord.hfsPlusFile.fileID, 
+			    	                      catRecord.hfsPlusFile.dataFork.extents, kDataFork);
+
+				/* HFSPlus resource fork */
+				CheckHFSPlusExtentRecords(GPtr, catRecord.hfsPlusFile.fileID, 
+			    	                      catRecord.hfsPlusFile.resourceFork.extents, kRsrcFork);
+			} else {
+				/* HFS data extent */
+				CheckHFSExtentRecords(GPtr, catRecord.hfsFile.fileID, 
+			    	                  catRecord.hfsFile.dataExtents, kDataFork);
+
+				/* HFS resource extent */
+				CheckHFSExtentRecords(GPtr, catRecord.hfsFile.fileID,
+				                      catRecord.hfsFile.rsrcExtents, kRsrcFork);
+			}
+		}
+
+		/* Access the next record */
+		err = GetBTreeRecord( GPtr->calculatedCatalogFCB, selCode, &catKey, &catRecord, &recordSize, &hint );
+	} while (err == noErr); 
+
+traverseExtents:
+	/* Traverse the extents btree */ 
+	selCode = 0x8001;	/* Get first record from BTree */
+	err = GetBTreeRecord(GPtr->calculatedExtentsFCB, selCode, &extentKey, &extentRecord, &recordSize, &hint);
+	if (err != noErr) {
+		goto out;
+	}
+	selCode = 1;	/* Get next record */
+	do {
+		if (isHFSPlus) {
+			CheckHFSPlusExtentRecords(GPtr, extentKey.hfsPlus.fileID, 
+			                          extentRecord.hfsPlus, extentKey.hfsPlus.forkType);
+		} else {
+			CheckHFSExtentRecords(GPtr, extentKey.hfs.fileID, extentRecord.hfs, 
+			                      extentKey.hfs.forkType);
+		}
+
+		/* Access the next record */
+		err = GetBTreeRecord(GPtr->calculatedExtentsFCB, selCode, &extentKey, &extentRecord, &recordSize, &hint);
+	} while (err == noErr); 
+
+out:
+	if (err == btNotFound) {
+		err = noErr;
+	}
+	return err;
+} /* FindOrigOverlapFiles */
+
+/* Function: PrintOverlapFiles
+ *
+ * Description: Print the information about all unique overlapping files.  
+ * 1. Sort the overlap extent in increasing order of fileID
+ * 2. For every unique fileID, prefix the string with fileID and find the
+ *    filename/path based on fileID.
+ *		If fileID > kHFSFirstUserCatalogNodeID, find path to file
+ *		Else, find name of the system file.
+ * 3. Print the new string.
+ * Note that the path is printed only for HFS Plus volumes and not for 
+ * plain HFS volumes.  This is done by not allocating buffer for finding
+ * file path.
+ *
+ * Input:
+ *	GPtr - Global scavenger structure pointer.
+ *
+ * Output:
+ *	nothing (void)
+ */
+void PrintOverlapFiles (SGlobPtr GPtr)
+{
+	OSErr err;
+	ExtentsTable **extentsTableH;
+	ExtentInfo *extentInfo;
+	unsigned int numOverlapExtents;
+	unsigned int buflen, filepathlen;
+	char *filepath = NULL;
+	char filenum[32];
+	UInt32 lastID = 0;
+	UInt32 bytesWritten;
+	Boolean printMsg;
+	Boolean	isHFSPlus;
+	int i;
+	
+	isHFSPlus = VolumeObjectIsHFSPlus();
+
+	extentsTableH = GPtr->overlappedExtents;
+	numOverlapExtents = (**extentsTableH).count;
+	
+	/* Sort the list according to file ID */
+	qsort((**extentsTableH).extentInfo, numOverlapExtents, sizeof(ExtentInfo), 
+		  CompareExtentFileID);
+
+	buflen = PATH_MAX * 4;
+	/* Allocate buffer to read data */
+	if (isHFSPlus) {
+		filepath = malloc (buflen);
+	}
+	
+	for (i = 0; i < numOverlapExtents; i++) {
+		extentInfo = &((**extentsTableH).extentInfo[i]);
+
+		/* Skip the same fileID */
+		if (lastID == extentInfo->fileID) {
+			continue;
+		}
+
+		lastID = extentInfo->fileID;
+		printMsg = false;
+
+		if (filepath) {
+			/* prefix with the file ID */
+			bytesWritten = sprintf (filepath, "%u ", extentInfo->fileID);
+			filepathlen = buflen - bytesWritten;
+	
+			if (extentInfo->fileID >= kHFSFirstUserCatalogNodeID) {
+				/* Lookup the file path */
+				err = GetFileNamePathByID (GPtr, extentInfo->fileID, (filepath + bytesWritten), &filepathlen, NULL, NULL, NULL);
+			} else {
+				/* Get system filename */
+			 	err = GetSystemFileName (extentInfo->fileID, (filepath + bytesWritten), &filepathlen);
+			}
+
+			if (err == noErr) {
+				/* print fileID, filepath */
+				PrintError(GPtr, E_OvlExt, 1, filepath);
+				printMsg = true;
+			}
+		}
+
+		if (printMsg == false) {
+			/* print only filenumber */
+			sprintf(filenum, "%u", extentInfo->fileID);
+			PrintError(GPtr, E_OvlExt, 1, filenum);
+		}
+	}
+
+	if (filepath) {
+		free (filepath);
+	}
+
+	return;
+} /* PrintOverlapFiles */
+
+/* Function: CompareExtentFileID
+ *
+ * Description: Compares the fileID from two ExtentInfo and return the
+ * comparison result. (since we have to arrange in ascending order)
+ *
+ * Input:
+ *	first and second - void pointers to ExtentInfo structure.
+ *
+ * Output:
+ *	>0 if first > second
+ * 	=0 if first == second
+ *	<0 if first < second
+ */
+static int CompareExtentFileID(const void *first, const void *second)
+{
+	return (((ExtentInfo *)first)->fileID - 
+			((ExtentInfo *)second)->fileID);
+} /* CompareExtentFileID */
