@@ -2,21 +2,22 @@
  * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
- *
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
- *
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * 
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
- *
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
+ * 
  * @APPLE_LICENSE_HEADER_END@
  */
 
@@ -58,8 +59,8 @@ HFSPlusCatalogKey gMetaDataDirKey = {
 
 /* private local routines */
 static int  GetPrivateDir(SGlobPtr gp, CatalogRecord *rec);
-static int  RecordOrphanINode(SGlobPtr gp, UInt32 parID, char * filename);
-static int  RecordBadLinkCount(SGlobPtr gp, UInt32 parID, char * filename,
+static int  RecordOrphanINode(SGlobPtr gp, UInt32 parID, unsigned char * filename);
+static int  RecordBadLinkCount(SGlobPtr gp, UInt32 parID, unsigned char * filename,
                                 int badcnt, int goodcnt);
 static void hash_insert(UInt32 linkID, int m, int n, struct IndirectLinkInfo *linkInfo);
 static struct IndirectLinkInfo * hash_search(UInt32 linkID, int m, int n, struct IndirectLinkInfo *linkInfo);
@@ -198,7 +199,7 @@ CheckHardLinks(void *cookie)
 	int prefixlen;
 	int result;
 	struct IndirectLinkInfo * linkInfo;
-	char filename[64];
+	unsigned char filename[64];
 
 	/* All done if no hard links exist. */
 	if (info == NULL || (info->privDirID == 0 && info->slotsUsed == 0))
@@ -248,17 +249,20 @@ CheckHardLinks(void *cookie)
 					filename, &len);
 		filename[len] = '\0';
 		
-		if ( 	(strstr(filename, HFS_DELETE_PREFIX) == filename) &&
-				(gp->calculatedVCB->vcbDriverWriteRef != -1) ) {
-				
+		/* Report Orphaned nodes in verify or on read-only volumes when:
+		 * 1. Volume was unmounted cleanly.
+		 * 2. fsck is being run in debug mode.
+		 */
+		if ((strstr((char *)filename, HFS_DELETE_PREFIX) == (char *)filename) &&
+			(gp->logLevel == kDebugLog)) {
 			RecordOrphanINode(gp, folderID, filename);
 			continue;		
 		}
 				
-		if (strstr(filename, HFS_INODE_PREFIX) != filename)
+		if (strstr((char *)filename, HFS_INODE_PREFIX) != (char *)filename)
 			continue;
 		
-		linkID = atol(&filename[prefixlen]);
+		linkID = atol((char *)&filename[prefixlen]);
 		linkCount = rec.hfsPlusFile.bsdInfo.special.linkCount;
 		linkInfo = hash_search(linkID, info->linkSlots, info->slotsUsed, info->linkInfo);
 		if (linkInfo) {
@@ -337,14 +341,14 @@ GetPrivateDir(SGlobPtr gp, CatalogRecord * rec)
  * Record a repair to delete an orphaned indirect node.
  */
 static int
-RecordOrphanINode(SGlobPtr gp, UInt32 parID, char* filename)
+RecordOrphanINode(SGlobPtr gp, UInt32 parID, unsigned char* filename)
 {
 	RepairOrderPtr p;
 	int n;
 	
 	PrintError(gp, E_UnlinkedFile, 1, filename);
 	
-	n = strlen(filename);
+	n = strlen((char *)filename);
 	p = AllocMinorRepairOrder(gp, n + 1);
 	if (p == NULL)
 		return (R_NoMem);
@@ -368,7 +372,7 @@ RecordOrphanINode(SGlobPtr gp, UInt32 parID, char* filename)
  * Record a repair to adjust an indirect node's link count.
  */
 static int
-RecordBadLinkCount(SGlobPtr gp, UInt32 parID, char * filename,
+RecordBadLinkCount(SGlobPtr gp, UInt32 parID, unsigned char * filename,
                    int badcnt, int goodcnt)
 {
 	RepairOrderPtr p;
@@ -381,7 +385,7 @@ RecordBadLinkCount(SGlobPtr gp, UInt32 parID, char * filename,
 	sprintf(badstr, "%d", badcnt);
 	PrintError(gp, E_BadValue, 2, goodstr, badstr);
 
-	n = strlen(filename);
+	n = strlen((char *)filename);
 	p = AllocMinorRepairOrder(gp, n + 1);
 	if (p == NULL)
 		return (R_NoMem);
