@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -42,13 +41,13 @@ Public routines:
 	BlockDeallocate
 					Deallocate a contiguous run of allocation blocks.
 
+Internal routines:
 	BlockAllocateAny
 					Find and allocate a contiguous range of blocks up to a given size.  The
 					first range of contiguous free blocks found are allocated, even if there
 					are fewer blocks than requested (and even if a contiguous range of blocks
 					of the given size exists elsewhere).
 
-Internal routines:
 	BlockMarkFree
 					Mark a contiguous range of blocks as free.  The corresponding
 					bits in the volume bitmap will be cleared.
@@ -137,7 +136,7 @@ static OSErr BlockMarkFree(
 /*
 ;________________________________________________________________________________
 ;
-; Routine:	   BlkAlloc
+; Routine:	   BlockAllocate
 ;
 ; Function:    Allocate space on a volume.	If contiguous allocation is requested,
 ;			   at least the requested number of bytes will be allocated or an
@@ -151,6 +150,10 @@ static OSErr BlockMarkFree(
 ;			   the volume's allocation block pointer will be used as a starting
 ;			   point.
 ;
+;				The function uses on-disk volume bitmap for allocation
+;				and updates it with newly allocated blocks.  It also 
+;				updates the in-memory volume bitmap.
+;
 ; Input Arguments:
 ;	 vcb			 - Pointer to SVCB for the volume to allocate space on
 ;	 fcb			 - Pointer to FCB for the file for which storage is being allocated
@@ -265,9 +268,11 @@ Exit:
 /*
 ;________________________________________________________________________________
 ;
-; Routine:	   BlkDealloc
+; Routine:	   BlockDeallocate
 ;
 ; Function:    Update the bitmap to deallocate a run of disk allocation blocks
+;	 The on-disk volume bitmap is read and updated; the in-memory volume bitmap 
+;	 is also updated.
 ;
 ; Input Arguments:
 ;	 vcb		- Pointer to SVCB for the volume to free space on
@@ -278,7 +283,8 @@ Exit:
 ;	 (result)	- Result code
 ;
 ; Side effects:
-;	 The volume bitmap is read and updated; the volume bitmap cache may be changed.
+;	 The on-disk volume bitmap is read and updated; the in-memory volume bitmap 
+;	 is also changed.
 ;
 ; Modification history:
 ;
@@ -379,7 +385,7 @@ static OSErr ReadBitmapBlock(
 		blockNum = bit / kBitsPerBlock;   /* block offset within bitmap */
 		blockNum += vcb->vcbVBMSt;        /* block within whole volume  */
 		
-		err = GetVolumeBlock(vcb, blockNum, kGetBlock, block);
+		err = GetVolumeBlock(vcb, blockNum, kGetBlock | kSkipEndianSwap, block);
 
 	} else {
 		// HFS+:  "bit" is the allocation block number that we are looking for  
@@ -403,7 +409,7 @@ static OSErr ReleaseBitmapBlock(
 	OSErr err;
 
 	if (vcb->vcbSignature == kHFSSigWord)
-		err = ReleaseVolumeBlock (vcb, block, options);
+		err = ReleaseVolumeBlock (vcb, block, options | kSkipEndianSwap);
 	else
 		err = ReleaseFileBlock (vcb->vcbAllocationFile, block, options);
 
@@ -422,6 +428,10 @@ Function:	Allocate a contiguous group of allocation blocks.  The
 			there are enough free blocks (though they may not be
 			contiguous, in which case this call will fail).
 
+			The function uses on-disk volume bitmap for allocation
+			and updates it with newly allocated blocks.  It also 
+			updates the in-memory volume bitmap.
+
 Inputs:
 	vcb				Pointer to volume where space is to be allocated
 	startingBlock	Preferred first block for allocation
@@ -484,6 +494,10 @@ Function:	Allocate one or more allocation blocks.  If there are fewer
 			allocated.  The caller guarantees that there is at least
 			one free block.
 
+			The function uses on-disk volume bitmap for allocation
+			and updates it with newly allocated blocks.  It also 
+			updates the in-memory volume bitmap.
+
 Inputs:
 	vcb				Pointer to volume where space is to be allocated
 	startingBlock	Preferred first block for allocation
@@ -504,7 +518,7 @@ static OSErr BlockAllocateAny(
 	UInt32			*actualNumBlocks)
 {
 	OSErr			err;
-	register UInt32	block;			//	current block number
+	register UInt32	block = 0;		//	current block number
 	register UInt32	currentWord;	//	Pointer to current word within bitmap block
 	register UInt32	bitMask;		//	Word with given bits already set (ready to OR in)
 	register UInt32	wordsLeft;		//	Number of words left in this bitmap block
@@ -535,7 +549,7 @@ static OSErr BlockAllocateAny(
 		wordIndexInBlock = (startingBlock & kBitsWithinBlockMask) / kBitsPerWord;
 		buffer += wordIndexInBlock;
 		wordsLeft = kWordsPerBlock - wordIndexInBlock;
-		currentWord = *buffer;
+		currentWord = SWAP_BE32(*buffer);
 		bitMask = kHighBitInWordMask >> (startingBlock & kBitsWithinWordMask);
 	}
 	
@@ -568,7 +582,7 @@ static OSErr BlockAllocateAny(
 
 				wordsLeft = kWordsPerBlock;
 			}
-			currentWord = *buffer;
+			currentWord = SWAP_BE32(*buffer);
 		}
 	}
 
@@ -605,7 +619,7 @@ static OSErr BlockAllocateAny(
 		//	Next bit
 		bitMask >>= 1;
 		if (bitMask == 0) {
-			*buffer = currentWord;					//	update value in bitmap
+			*buffer = SWAP_BE32(currentWord);	// update value in bitmap
 			
 			//	Next word
 			bitMask = kHighBitInWordMask;
@@ -624,14 +638,17 @@ static OSErr BlockAllocateAny(
 
 				wordsLeft = kWordsPerBlock;
 			}
-			currentWord = *buffer;
+			currentWord = SWAP_BE32(*buffer);
 		}
 	}
-	*buffer = currentWord;							//	update the last change
-
+	*buffer = SWAP_BE32(currentWord);	// update the last change
+	
 Exit:
 	if (err == noErr) {
 		*actualNumBlocks = block - *actualStartBlock;
+		
+		/* Update the in-memory copy of bitmap */
+		(void) CaptureBitmapBits (*actualStartBlock, *actualNumBlocks);
 	}
 	else {
 		*actualStartBlock = 0;
@@ -652,8 +669,12 @@ _______________________________________________________________________
 Routine:	BlockMarkAllocated
 
 Function:	Mark a contiguous group of blocks as allocated (set in the
-			bitmap).  It assumes those bits are currently marked
-			deallocated (clear in the bitmap).
+			bitmap).  The function sets the bit independent of the 
+			previous state (set/clear) of the bit.
+
+			The function uses on-disk volume bitmap for allocation
+			and updates it with newly allocated blocks.  It also 
+			updates the in-memory volume bitmap.
 
 Inputs:
 	vcb				Pointer to volume where space is to be allocated
@@ -676,6 +697,9 @@ static OSErr BlockMarkAllocated(
 	BlockDescriptor bd = {0};
 	OptionBits  relOpt = kReleaseBlock;
 
+	UInt32 saveNumBlocks = numBlocks;
+	UInt32 saveStartingBlock = startingBlock;
+
 	//
 	//	Pre-read the bitmap block containing the first word of allocation
 	//
@@ -711,13 +735,13 @@ static OSErr BlockMarkAllocated(
 			bitMask &= ~(kAllBitsSetInWord >> (firstBit + numBits));	//	turn off bits after last
 		}
 #if DEBUG_BUILD
-		if ((*currentWord & bitMask) != 0) {
+		if ((*currentWord & SWAP_BE32(bitMask)) != 0) {
 			DebugStr("\pFATAL: blocks already allocated!");
 			err = fsDSIntErr;
 			goto Exit;
 		}
 #endif
-		*currentWord |= bitMask;					//	set the bits in the bitmap
+		*currentWord |= SWAP_BE32(bitMask);				//	set the bits in the bitmap
 		numBlocks -= numBits;						//	adjust number of blocks left to allocate
 
 		++currentWord;								//	move to next word
@@ -754,7 +778,7 @@ static OSErr BlockMarkAllocated(
 			goto Exit;
 		}
 #endif
-		*currentWord = bitMask;
+		*currentWord = SWAP_BE32(bitMask);
 		numBlocks -= kBitsPerWord;
 
 		++currentWord;								//	move to next word
@@ -785,17 +809,20 @@ static OSErr BlockMarkAllocated(
 			wordsLeft = kWordsPerBlock;
 		}
 #if DEBUG_BUILD
-		if ((*currentWord & bitMask) != 0) {
+		if ((*currentWord & SWAP_BE32(bitMask)) != 0) {
 			DebugStr("\pFATAL: blocks already allocated!");
 			err = fsDSIntErr;
 			goto Exit;
 		}
 #endif
-		*currentWord |= bitMask;						//	set the bits in the bitmap
+		*currentWord |= SWAP_BE32(bitMask);						//	set the bits in the bitmap
 
 		//	No need to update currentWord or wordsLeft
 	}
 
+	/* Update the in-memory copy of the volume bitmap */
+	(void) CaptureBitmapBits(saveStartingBlock, saveNumBlocks);
+
 Exit:
 	if (bd.buffer != NULL)
 		(void) ReleaseBitmapBlock(vcb, relOpt, &bd);
@@ -811,8 +838,11 @@ _______________________________________________________________________
 Routine:	BlockMarkFree
 
 Function:	Mark a contiguous group of blocks as free (clear in the
-			bitmap).  It assumes those bits are currently marked
-			allocated (set in the bitmap).
+			bitmap).  The function clears the bit independent of the 
+			previous state (set/clear) of the bit.
+
+			This function uses the on-disk bitmap and also updates 
+			the in-memory bitmap with the deallocated blocks
 
 Inputs:
 	vcb				Pointer to volume where space is to be freed
@@ -835,6 +865,9 @@ static OSErr BlockMarkFree(
 	BlockDescriptor bd = {0};
 	OptionBits  relOpt = kReleaseBlock;
 
+	UInt32 saveNumBlocks = numBlocks;
+	UInt32 saveStartingBlock = startingBlock;
+
 	//
 	//	Pre-read the bitmap block containing the first word of allocation
 	//
@@ -870,13 +903,13 @@ static OSErr BlockMarkFree(
 			bitMask &= ~(kAllBitsSetInWord >> (firstBit + numBits));	//	turn off bits after last
 		}
 #if DEBUG_BUILD
-		if ((*currentWord & bitMask) != bitMask) {
+		if ((*currentWord & SWAP_BE32(bitMask)) != SWAP_BE32(bitMask)) {
 			DebugStr("\pFATAL: blocks not allocated!");
 			err = fsDSIntErr;
 			goto Exit;
 		}
 #endif
-		*currentWord &= ~bitMask;					//	clear the bits in the bitmap
+		*currentWord &= SWAP_BE32(~bitMask);			// clear the bits in the bitmap
 		numBlocks -= numBits;						//	adjust number of blocks left to free
 
 		++currentWord;								//	move to next word
@@ -943,16 +976,19 @@ static OSErr BlockMarkFree(
 			wordsLeft = kWordsPerBlock;
 		}
 #if DEBUG_BUILD
-		if ((*currentWord & bitMask) != bitMask) {
+		if ((*currentWord & SWAP_BE32(bitMask)) != SWAP_BE32(bitMask)) {
 			DebugStr("\pFATAL: blocks not allocated!");
 			err = fsDSIntErr;
 			goto Exit;
 		}
 #endif
-		*currentWord &= ~bitMask;						//	clear the bits in the bitmap
+		*currentWord &= SWAP_BE32(~bitMask);						//	clear the bits in the bitmap
 
 		//	No need to update currentWord or wordsLeft
 	}
+	
+	/* Update the in-memory copy of the volume bitmap */
+	(void) ReleaseBitmapBits(saveStartingBlock, saveNumBlocks);
 
 Exit:
 	if (bd.buffer != NULL)
@@ -1100,7 +1136,7 @@ static OSErr BlockFindContiguous(
 		currentWord		= buffer + wordIndexInBlock;
 				
 		wordsLeft		= wordIndexInBlock;
-		tempWord		= *currentWord;
+		tempWord		= SWAP_BE32(*currentWord);
 		bitMask			= kHighBitInWordMask >> ( currentBlock & kBitsWithinWordMask );
 		currentSector	= currentBlock / kBitsPerBlock;
 	}
@@ -1150,7 +1186,7 @@ static OSErr BlockFindContiguous(
 					
 				wordsLeft	= ( currentBlock & kBitsWithinBlockMask ) / kBitsPerWord;
 				currentWord	= buffer + wordsLeft;
-				tempWord	= *currentWord;
+				tempWord	= SWAP_BE32(*currentWord);
 				bitMask		= kHighBitInWordMask >> ( currentBlock & kBitsWithinWordMask );
 
 				continue;												//	Back to the while loop
@@ -1187,7 +1223,7 @@ NextWord:
 				wordsLeft		= kWordsPerBlock - 1;
 			}
 			
-			tempWord = *currentWord;							//	Grab the current word
+			tempWord = SWAP_BE32(*currentWord);							//	Grab the current word
 
 			//
 			//	If we found a whole word of free blocks, quickly skip over it.
@@ -1250,7 +1286,7 @@ NextWord:
 
 		wordIndexInBlock = (currentBlock & kBitsWithinBlockMask) / kBitsPerWord;
 		currentWord		= buffer + wordIndexInBlock;
-		tempWord		= *currentWord;
+		tempWord		= SWAP_BE32(*currentWord);
 		wordsLeft		= kWordsPerBlock - wordIndexInBlock;
 		bitMask			= kHighBitInWordMask >> (currentBlock & kBitsWithinWordMask);
 	}
@@ -1294,7 +1330,7 @@ NextWord:
 					currentWord		= buffer;
 					wordsLeft		= kWordsPerBlock;
 				}
-				tempWord = *currentWord;			//	grab the current word
+				tempWord = SWAP_BE32(*currentWord);	// grab the current word
 			}
 		}
 	}
