@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 2002 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -175,11 +174,9 @@ OSErr	RebuildCatalogBTree( SGlobPtr theSGlobPtr )
 		myVCBPtr->vcbBackupDate = myVHBPtr->backupDate;	
 		myVCBPtr->vcbCatalogFile->fcbClumpSize = myVHBPtr->catalogFile.clumpSize;
 
-		//	verify volume attribute flags
-		if ( ((UInt16)myVHBPtr->attributes & VAtrb_Msk) == 0 )
-			myVCBPtr->vcbAttributes = (UInt16)myVHBPtr->attributes;
-		else 
-			myVCBPtr->vcbAttributes = VAtrb_DFlt;
+		// 3882639: Removed check for volume attributes in HFS Plus 
+		myVCBPtr->vcbAttributes = myVHBPtr->attributes;
+
 		CopyMemory( myVHBPtr->finderInfo, myVCBPtr->vcbFinderInfo, sizeof(myVCBPtr->vcbFinderInfo) );
 	}
 	else
@@ -361,7 +358,7 @@ static OSErr CreateNewCatalogBTree( SGlobPtr theSGlobPtr )
 	BTreeControlBlock *		myBTreeCBPtr;
 	SVCB *					myVCBPtr;
 	SFCB *					myFCBPtr;
-	UInt32 					myBytesUsed;
+	UInt32 					myBytesUsed = 0;
 	UInt32 					myMapNodeCount;
 	FSSize					myNewEOF;
 	BTHeaderRec				myHeaderRec;
