@@ -1,23 +1,22 @@
 /*
- * Copyright (c) 1999-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999-2003, 2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -283,14 +282,22 @@ OSErr	GetBTreeHeader( SGlobPtr GPtr, SFCB *fcb, BTHeaderRec *header )
 	err = GetFileBlock(fcb, kHeaderNodeNum, kGetBlock, &block);
 	ReturnIfError(err);
 
-	SWAP_BT_NODE(&block, (fcb->fcbVolume->vcbSignature == kHFSPlusSigWord),
-		fcb->fcbFileID, 3);
+	err = hfs_swap_BTNode(&block, fcb, kSwapBTNodeHeaderRecordOnly);
+	if (err != noErr)
+	{
+		(void) ReleaseFileBlock(fcb, &block, kReleaseBlock | kTrashBlock);
+		return err;
+	}
 
 	headerRec = (BTHeaderRec *)((char*)block.buffer + sizeof(BTNodeDescriptor));
 	CopyMemory(headerRec, header, sizeof(BTHeaderRec));
 
-	SWAP_BT_NODE(&block, (fcb->fcbVolume->vcbSignature == kHFSPlusSigWord),
-		fcb->fcbFileID, 3);
+	err = hfs_swap_BTNode(&block, fcb, kSwapBTNodeHeaderRecordOnly);
+	if (err != noErr)
+	{
+		(void) ReleaseFileBlock(fcb, &block, kReleaseBlock | kTrashBlock);
+		return err;
+	}
 	
 	err = ReleaseFileBlock (fcb, &block, kReleaseBlock);
 	ReturnIfError(err);
