@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -44,7 +43,7 @@ static	OSErr	RcdMDBEmbededVolDescriptionErr( SGlobPtr GPtr, OSErr type, HFSMaste
 
 static	OSErr	CheckNodesFirstOffset( SGlobPtr GPtr, BTreeControlBlock *btcb );
 
-static	Boolean	ExtentInfoExists( ExtentsTable **extentsTableH, ExtentInfo *extentInfo );
+static	Boolean	ExtentInfoExists( ExtentsTable **extentsTableH, ExtentInfo *extentInfo, Boolean *doesFileExist );
 
 static OSErr	ScavengeVolumeType( SGlobPtr GPtr, HFSMasterDirectoryBlock *mdb, UInt32 *volumeType );
 static OSErr	SeekVolumeHeader( SGlobPtr GPtr, UInt64 startSector, UInt32 numSectors, UInt64 *vHSector );
@@ -107,8 +106,10 @@ CheckIfJournaled(SGlobPtr GPtr)
 		// even if journaling is enabled for this volume, we'll return
 		// false if it wasn't unmounted cleanly and it was previously
 		// mounted by someone that doesn't know about journaling.
-		if (   vhp->lastMountedVersion != kHFSJMountVersion
-			&& (vhp->attributes & kHFSVolumeUnmountedMask) == 0) {
+		// or if lastMountedVersion is kFSKMountVersion
+		if ( vhp->lastMountedVersion == kFSKMountVersion || 
+			((vhp->lastMountedVersion != kHFSJMountVersion) && 
+			(vhp->attributes & kHFSVolumeUnmountedMask) == 0)) {
 			result = 0;
 		}
 	} else {
@@ -167,8 +168,33 @@ CheckForClean(SGlobPtr GPtr, Boolean markClean)
 	if ( VolumeObjectIsHFSPlus( ) ) {
 		vhp = (HFSPlusVolumeHeader *) block.buffer;
 		
-		result = (vhp->attributes & kHFSVolumeUnmountedMask) != 0;
-		if (markClean && (result == 0)) {
+		result = 1;  // clean unmount
+		if ( ((vhp->attributes & kHFSVolumeUnmountedMask) == 0) )
+			result = 0;  // dirty unmount
+
+		// if we are to mark volume clean and lastMountedVersion shows journaled we invalidate
+		// the journal by setting lastMountedVersion to 'fsck'
+		if ( markClean && ((vhp->lastMountedVersion == kHFSJMountVersion) || 
+						   (vhp->lastMountedVersion == kFSKMountVersion)) ) {
+			result = 0;  // force dirty unmount path
+			vhp->lastMountedVersion = kFSCKMountVersion;
+		}
+		
+		if (vhp->lastMountedVersion == kFSKMountVersion) {
+			GPtr->JStat |= S_BadJournal;
+			if ( GPtr->logLevel >= kDebugLog ) 
+				printf ("\t%s - found bad journal signature \n", __FUNCTION__);
+			/* XXX This is not correct error code for bad journal.  
+			 * E_InvalidVolumeHeader is negative error.  Change ErrCode
+			 * to non-negative, else it might indicate non-fixable error
+			 * in scavenge process 
+			 */
+			RcdError (GPtr, E_InvalidVolumeHeader);
+			GPtr->ErrCode = -E_InvalidVolumeHeader;
+			result = 0;	//dirty unmount
+		}
+		
+		if ( markClean && (result == 0) ) {
 			vhp->attributes |= kHFSVolumeUnmountedMask;
 			rbOptions = kForceWriteBlock;
 		}
@@ -287,6 +313,13 @@ OSErr IVChk( SGlobPtr GPtr )
 
 			GetVolumeObjectBlockNum( &blockNum ); // get the new Volume header block number
 		}
+		else {
+			if ( GPtr->logLevel >= kDebugLog )
+				printf( "\t%s - bad volume header - err %d \n", __FUNCTION__, err );
+			WriteError( GPtr, E_InvalidVolumeHeader, 1, 0 );
+			err = E_InvalidVolumeHeader;
+			goto ReleaseAndBail;
+		}
 	}
 
 	totalSectors = ( VolumeObjectIsEmbeddedHFSPlus( ) ) ? myVOPtr->totalEmbeddedSectors : myVOPtr->totalDeviceSectors;
@@ -1184,8 +1217,20 @@ OSErr	CreateExtendedAllocationsFCB( SGlobPtr GPtr )
 		err = CheckFileExtents( GPtr, kHFSAllocationFileID, 0, (void *)fcb->fcbExtents32, &numABlks );
 		if (err) goto exit;
 
-		(void) SetFileBlockSize (fcb, vcb->vcbBlockSize);
-
+		//
+		// The allocation file will get processed in whole allocation blocks, or
+		// maximal-sized cache blocks, whichever is smaller.  This means the cache
+		// doesn't need to cope with buffers that are larger than a cache block.
+		//
+		// The definition of CACHE_IOSIZE below matches the definition in fsck_hfs.c.
+		// If you change it here, then change it there, too.  Or else put it in some
+		// common header file.
+		#define CACHE_IOSIZE	32768
+		if (vcb->vcbBlockSize < CACHE_IOSIZE)
+			(void) SetFileBlockSize (fcb, vcb->vcbBlockSize);
+		else
+			(void) SetFileBlockSize (fcb, CACHE_IOSIZE);
+	
 		if ( volumeHeader->allocationFile.totalBlocks != numABlks )
 		{
 			RcdError( GPtr, E_CatPEOF );
@@ -1545,7 +1590,7 @@ resumeAtParent:
 			valence = isHFSPlus == true ? record2.hfsPlusFolder.valence : (UInt32)record2.hfsFolder.valence;
 
 			if ( valence != dprP->offspringIndex -1 ) 				/* check its valence */
-				if ( result = RcdValErr( GPtr, E_DirVal, dprP->offspringIndex -1, valence, dprP->parentDirID ) )
+				if ( ( result = RcdValErr( GPtr, E_DirVal, dprP->offspringIndex -1, valence, dprP->parentDirID ) ) )
 					return( result );
 
 			GPtr->DirLevel--;										/* move up a level */			
@@ -1564,19 +1609,19 @@ resumeAtParent:
 	//	verify directory and file counts (all nonfatal, repairable errors)
 	//
 	if (!isHFSPlus && (rtdirCnt != calculatedVCB->vcbNmRtDirs)) /* check count of dirs in root */
-		if ( result = RcdValErr(GPtr,E_RtDirCnt,rtdirCnt,calculatedVCB->vcbNmRtDirs,0) )
+		if ( ( result = RcdValErr(GPtr,E_RtDirCnt,rtdirCnt,calculatedVCB->vcbNmRtDirs,0) ) )
 			return( result );
 
 	if (!isHFSPlus && (rtfilCnt != calculatedVCB->vcbNmFls)) /* check count of files in root */
-		if ( result = RcdValErr(GPtr,E_RtFilCnt,rtfilCnt,calculatedVCB->vcbNmFls,0) )
+		if ( ( result = RcdValErr(GPtr,E_RtFilCnt,rtfilCnt,calculatedVCB->vcbNmFls,0) ) )
 			return( result );
 
 	if (dirCnt != calculatedVCB->vcbFolderCount) /* check count of dirs in volume */
-		if ( result = RcdValErr(GPtr,E_DirCnt,dirCnt,calculatedVCB->vcbFolderCount,0) )
+		if ( ( result = RcdValErr(GPtr,E_DirCnt,dirCnt,calculatedVCB->vcbFolderCount,0) ) )
 			return( result );
 		
 	if (filCnt != calculatedVCB->vcbFileCount) /* check count of files in volume */
-		if ( result = RcdValErr(GPtr,E_FilCnt,filCnt,calculatedVCB->vcbFileCount,0) )
+		if ( ( result = RcdValErr(GPtr,E_FilCnt,filCnt,calculatedVCB->vcbFileCount,0) ) )
 			return( result );
 
 	return( noErr );
@@ -1771,7 +1816,7 @@ Output:		int		-	function result:
 ------------------------------------------------------------------------------*/
 
 static int 
-CheckAttributeRecord(SGlobPtr GPtr, const AttributeKey *key, const HFSPlusAttrRecord *rec, UInt16 reclen)
+CheckAttributeRecord(SGlobPtr GPtr, const HFSPlusAttrKey *key, const HFSPlusAttrRecord *rec, UInt16 reclen)
 {
 	int result = 0;
 	Boolean isHFSPlus;
@@ -1789,7 +1834,7 @@ CheckAttributeRecord(SGlobPtr GPtr, const AttributeKey *key, const HFSPlusAttrRe
 	/* Convert unicode attribute name to char */
 	(void) utf_encodestr(key->attrName, key->attrNameLen * 2, attrName, &len);
 	attrName[len] = '\0';
-	printf ("%s(%s,%d): fileID=%d AttributeName=%s\n", __FUNCTION__, __FILE__, __LINE__, key->cnid, attrName);
+	printf ("%s(%s,%d): fileID=%d AttributeName=%s\n", __FUNCTION__, __FILE__, __LINE__, key->fileID, attrName);
 #endif
 	
 	/* 3984119 - Do not include extended attributes for file IDs less
@@ -1797,23 +1842,23 @@ CheckAttributeRecord(SGlobPtr GPtr, const AttributeKey *key, const HFSPlusAttrRe
 	 * in prime modulus checksum.  These file IDs do not have 
 	 * any catalog record
 	 */
-	if ((key->cnid < kHFSFirstUserCatalogNodeID) && 
-	    (key->cnid != kHFSRootFolderID)) {
+	if ((key->fileID < kHFSFirstUserCatalogNodeID) && 
+	    (key->fileID != kHFSRootFolderID)) {
 		goto out;
 	}
 	
-	if (GPtr->lastAttrFileID.fileID != key->cnid) {
+	if (GPtr->lastAttrFileID.fileID != key->fileID) {
 		/* fsck is seeing this fileID in Attribute BTree for first time */
-		GPtr->lastAttrFileID.fileID = key->cnid;
+		GPtr->lastAttrFileID.fileID = key->fileID;
 		GPtr->lastAttrFileID.hasSecurity = false;
 		
 		if (!bcmp(key->attrName, GPtr->securityAttrName, GPtr->securityAttrLen)) {
 			/* file has both extended attribute and ACL */
-			RecordXAttrBits(GPtr, kHFSHasAttributesMask | kHFSHasSecurityMask, key->cnid, kCalculatedAttributesRefNum);
+			RecordXAttrBits(GPtr, kHFSHasAttributesMask | kHFSHasSecurityMask, key->fileID, kCalculatedAttributesRefNum);
 			GPtr->lastAttrFileID.hasSecurity = true;
 		} else {
 			/* file only has extended attribute */
-			RecordXAttrBits(GPtr, kHFSHasAttributesMask, key->cnid, kCalculatedAttributesRefNum);
+			RecordXAttrBits(GPtr, kHFSHasAttributesMask, key->fileID, kCalculatedAttributesRefNum);
 		}
 	} else {
 		/* fsck has seen this fileID before */
@@ -1821,7 +1866,7 @@ CheckAttributeRecord(SGlobPtr GPtr, const AttributeKey *key, const HFSPlusAttrRe
 			/* If GPtr->lastAttrFileID did not have security AND current xattr is ACL, 
 			 * we ONLY record ACL (as we have already recorded extended attribute
 			 */
-			RecordXAttrBits(GPtr, kHFSHasSecurityMask, key->cnid, kCalculatedAttributesRefNum);
+			RecordXAttrBits(GPtr, kHFSHasSecurityMask, key->fileID, kCalculatedAttributesRefNum);
 			GPtr->lastAttrFileID.hasSecurity = true;
 		}
 	}
@@ -1865,7 +1910,7 @@ OSErr AttrBTChk( SGlobPtr GPtr )
 
 	err = BTCheck( GPtr, kCalculatedAttributesRefNum, (CheckLeafRecordProcPtr)CheckAttributeRecord);
 	ReturnIfError( err );														//	invalid attributes file BTree
-	
+
 	//	compare the attributes prime buckets calculated from catalog btree and attribute btree 
 	err = ComparePrimeBuckets(GPtr, kHFSHasAttributesMask);
 	ReturnIfError( err );
@@ -2089,7 +2134,7 @@ OSErr VInfoChk( SGlobPtr GPtr )
 	UInt16					recSize;
 	Boolean					isHFSPlus;
 	UInt32					hint;
-	UInt32					maxClump;
+	UInt64					maxClump;
 	SVCB					*vcb;
 	VolumeObjectPtr			myVOPtr;
 	CatalogRecord			record;
@@ -2187,18 +2232,15 @@ OSErr VInfoChk( SGlobPtr GPtr )
 		alternateVolumeHeader = (HFSPlusVolumeHeader *) altBlock.buffer;
 		volumeHeader = (HFSPlusVolumeHeader *) priBlock.buffer;
 	
-		maxClump = (vcb->vcbTotalBlocks / 4) * vcb->vcbBlockSize; /* max clump = 1/4 volume size */
+		maxClump = (UInt64) (vcb->vcbTotalBlocks / 4) * vcb->vcbBlockSize; /* max clump = 1/4 volume size */
 
 		//	check out creation and last mod dates
 		vcb->vcbCreateDate	= alternateVolumeHeader->createDate;	// use creation date in alt MDB
 		vcb->vcbModifyDate	= volumeHeader->modifyDate;		// don't change last mod date
 		vcb->vcbCheckedDate	= volumeHeader->checkedDate;		// don't change checked date
 
-		//	verify volume attribute flags
-		if ( ((UInt16)volumeHeader->attributes & VAtrb_Msk) == 0 )
-			vcb->vcbAttributes = (UInt16)volumeHeader->attributes;
-		else 
-			vcb->vcbAttributes = VAtrb_DFlt;
+		// 3882639: Removed check for volume attributes in HFS Plus 
+		vcb->vcbAttributes = volumeHeader->attributes;
 	
 		//	verify allocation map ptr
 		if ( volumeHeader->nextAllocation < vcb->vcbTotalBlocks )
@@ -2234,9 +2276,14 @@ OSErr VInfoChk( SGlobPtr GPtr )
 		if ( vcb->vcbDataClumpSize > kMaxClumpSize )
 			vcb->vcbDataClumpSize = vcb->vcbBlockSize;	/* for very large volumes, just use 1 allocation block */
 
-		//	verify next CNode ID
-		if ( (volumeHeader->nextCatalogID > vcb->vcbNextCatalogID) && 
-			 (volumeHeader->nextCatalogID <= (vcb->vcbNextCatalogID + 4096)) )
+		/* Verify next CNode ID.
+		 * If volumeHeader->nextCatalogID < vcb->vcbNextCatalogID, probably 
+		 * nextCatalogID has wrapped around.
+		 * If volumeHeader->nextCatalogID > vcb->vcbNextCatalogID, probably 
+		 * many files were created and deleted, followed by no new file 
+		 * creation.
+		 */
+		if ( (volumeHeader->nextCatalogID > vcb->vcbNextCatalogID) ) 
 			vcb->vcbNextCatalogID = volumeHeader->nextCatalogID;
 			
 		//TBD location and unicode? volumename
@@ -2267,7 +2314,7 @@ OSErr VInfoChk( SGlobPtr GPtr )
 		else if ( ((alternateVolumeHeader->catalogFile.clumpSize % vcb->vcbBlockSize) == 0) && 
 				  (alternateVolumeHeader->catalogFile.clumpSize <= maxClump) )
 			vcb->vcbCatalogFile->fcbClumpSize = alternateVolumeHeader->catalogFile.clumpSize;
-		else
+		else 
 			vcb->vcbCatalogFile->fcbClumpSize = 
 			(alternateVolumeHeader->catalogFile.extents[0].blockCount * vcb->vcbBlockSize);
 			
@@ -2312,7 +2359,7 @@ OSErr VInfoChk( SGlobPtr GPtr )
 		alternateMDB = (HFSMasterDirectoryBlock	*) altBlock.buffer;
 		mdbP = (HFSMasterDirectoryBlock	*) priBlock.buffer;
 
-		maxClump = (vcb->vcbTotalBlocks / 4) * vcb->vcbBlockSize; /* max clump = 1/4 volume size */
+		maxClump = (UInt64) (vcb->vcbTotalBlocks / 4) * vcb->vcbBlockSize; /* max clump = 1/4 volume size */
 
 		//	check out creation and last mod dates
 		vcb->vcbCreateDate	= alternateMDB->drCrDate;		/* use creation date in alt MDB */	
@@ -2591,7 +2638,6 @@ OSErr	CheckFileExtents( SGlobPtr GPtr, UInt32 fileNumber, UInt8 forkType,
 	
 			if ( extentBlockCount == 0 )
 				break;
-
 			err = CaptureBitmapBits(extentStartBlock, extentBlockCount);
 			if (err == E_OvlExt) {
 				err = AddExtentToOverlapList(GPtr, fileNumber, extentStartBlock, extentBlockCount, forkType);
@@ -2695,13 +2741,10 @@ OSErr	AddExtentToOverlapList( SGlobPtr GPtr, HFSCatalogNodeID fileNumber, UInt32
 	UInt32			newHandleSize;
 	ExtentInfo		extentInfo;
 	ExtentsTable	**extentsTableH;
-	char fileno[32];
+	Boolean doesFileExist = false;
 	
-	sprintf(fileno, "%ud", fileNumber);
-	PrintError(GPtr, E_OvlExt, 1, fileno);
-	GPtr->VIStat |= S_OverlappingExtents;
-	
-	extentInfo.fileNumber	= fileNumber;
+	ClearMemory(&extentInfo, sizeof(extentInfo));
+	extentInfo.fileID		= fileNumber;
 	extentInfo.startBlock	= extentStartBlock;
 	extentInfo.blockCount	= extentBlockCount;
 	extentInfo.forkType		= forkType;
@@ -2716,7 +2759,7 @@ OSErr	AddExtentToOverlapList( SGlobPtr GPtr, HFSCatalogNodeID fileNumber, UInt32
 	{
 		extentsTableH	= GPtr->overlappedExtents;
 
-		if ( ExtentInfoExists( extentsTableH, &extentInfo ) == true )
+		if ( ExtentInfoExists( extentsTableH, &extentInfo, &doesFileExist ) == true )
 			return( noErr );
 
 		//	Grow the Extents table for a new entry.
@@ -2724,9 +2767,16 @@ OSErr	AddExtentToOverlapList( SGlobPtr GPtr, HFSCatalogNodeID fileNumber, UInt32
 		SetHandleSize( (Handle)extentsTableH, newHandleSize );
 	}
 
+	/* Print the list of overlapping extents after catalog BTree check to
+	 * avoid deadlocks from cache read
+	 */
+	
 	//	Copy the new extents into the end of the table
 	CopyMemory( &extentInfo, &((**extentsTableH).extentInfo[(**extentsTableH).count]), sizeof(ExtentInfo) );
 	
+	// 	Update the overlap extent bit
+	GPtr->VIStat |= S_OverlappingExtents;
+
 	//	Update the extent table count
 	(**extentsTableH).count++;
 	
@@ -2734,17 +2784,21 @@ OSErr	AddExtentToOverlapList( SGlobPtr GPtr, HFSCatalogNodeID fileNumber, UInt32
 }
 
 
-static	Boolean	ExtentInfoExists( ExtentsTable **extentsTableH, ExtentInfo *extentInfo )
+static	Boolean	ExtentInfoExists( ExtentsTable **extentsTableH, ExtentInfo *extentInfo, Boolean *doesFileExist )
 {
 	UInt32		i;
 	ExtentInfo	*aryExtentInfo;
 	
+	*doesFileExist = false;
+
 	for ( i = 0 ; i < (**extentsTableH).count ; i++ )
 	{
 		aryExtentInfo	= &((**extentsTableH).extentInfo[i]);
 		
-		if ( extentInfo->fileNumber == aryExtentInfo->fileNumber )
+		if ( extentInfo->fileID == aryExtentInfo->fileID )
 		{
+			*doesFileExist = true;
+
 			if (	(extentInfo->startBlock == aryExtentInfo->startBlock)	&& 
 					(extentInfo->blockCount == aryExtentInfo->blockCount)	&&
 					(extentInfo->forkType	== aryExtentInfo->forkType)		)
