@@ -1,23 +1,22 @@
 /*
- * Copyright (c) 1999-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999-2003, 2005 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -135,6 +134,7 @@ BTCheck(SGlobPtr GPtr, short refNum, CheckLeafRecordProcPtr checkLeafRecord)
 	short			index;
 	UInt16			recSize;
 	UInt8			parKey[ kMaxKeyLength + 2 + 2 ];
+	Boolean			hasParKey = false;	
 	UInt8			*dataPtr;
 	STPR			*tprP;
 	STPR			*parentP;
@@ -171,7 +171,7 @@ BTCheck(SGlobPtr GPtr, short refNum, CheckLeafRecordProcPtr checkLeafRecord)
 	result = GetNode( calculatedBTCB, kHeaderNodeNum, &node );
 	if ( result != noErr )
 	{
-		if ( result == fsBTInvalidNodeErr )	/* CheckNode failed */
+		if ( result == fsBTInvalidNodeErr )	/* hfs_swap_BTNode failed */
 		{
 			RcdError( GPtr, E_BadNode );
 			result	= E_BadNode;
@@ -277,7 +277,7 @@ BTCheck(SGlobPtr GPtr, short refNum, CheckLeafRecordProcPtr checkLeafRecord)
 		result = GetNode( calculatedBTCB, nodeNum, &node );
 		if ( result != noErr )
 		{
-			if ( result == fsBTInvalidNodeErr )	/* CheckNode failed */
+			if ( result == fsBTInvalidNodeErr )	/* hfs_swap_BTNode failed */
 			{
 				RcdError( GPtr, E_BadNode );
 				result	= E_BadNode;
@@ -369,7 +369,12 @@ BTCheck(SGlobPtr GPtr, short refNum, CheckLeafRecordProcPtr checkLeafRecord)
 				goto exit;
 			}
 				
-			if ( parKey[0] != 0 )
+			/* If we saved the first key in the parent (index) node 
+			 * in past, use it to compare with the key of the first 
+			 * record in the current node.  This check should be 
+			 * performed for all nodes except the root node.
+			 */
+			if ( hasParKey == true )
 			{
 				GetRecordByIndex( (BTreeControlBlock *)calculatedBTCB, nodeDescP, 0, &keyPtr, &dataPtr, &recSize );
 				if ( CompareKeys( (BTreeControlBlockPtr)calculatedBTCB, (BTreeKey *)parKey, keyPtr ) != 0 )
@@ -430,6 +435,7 @@ BTCheck(SGlobPtr GPtr, short refNum, CheckLeafRecordProcPtr checkLeafRecord)
 						? keyPtr->length16 + sizeof(UInt16)
 						: keyPtr->length8 + sizeof(UInt8);
 			CopyMemory(keyPtr, parKey, keyLen);
+			hasParKey = true;
 				
 			tprP->TPRNodeN = nodeNum;
 			tprP->TPRRIndx = -1;
@@ -556,7 +562,7 @@ int BTMapChk( SGlobPtr GPtr, short fileRefNum )
 		result = GetNode( calculatedBTCB, nodeNum, &node );
 		if ( result != noErr )
 		{
-			if ( result == fsBTInvalidNodeErr )	/* CheckNode failed */
+			if ( result == fsBTInvalidNodeErr )	/* hfs_swap_BTNode failed */
 			{
 				RcdError( GPtr, E_BadNode );
 				result	= E_BadNode;
