@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 2000 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.1 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -105,7 +104,7 @@ int isdirty(int fs, struct bootblock *boot, int fat)
 
        buffer = malloc(boot->BytesPerSec);
        if (buffer == NULL) {
-               perror("No space for FAT sector");
+               perr("No space for FAT sector");
                return 1;               /* Assume it was dirty */
        }
 
@@ -113,12 +112,12 @@ int isdirty(int fs, struct bootblock *boot, int fat)
        offset *= boot->BytesPerSec;
 
        if (lseek(fs, offset, SEEK_SET) != offset) {
-               perror("Unable to read FAT");
+               perr("Unable to read FAT");
                goto ERROR;
        }
 
        if (deblock_read(fs, buffer, boot->BytesPerSec) != boot->BytesPerSec) {
-               perror("Unable to read FAT");
+               perr("Unable to read FAT");
                goto ERROR;
        }
 
@@ -233,13 +232,13 @@ static int _readchunk(int fd, struct bootblock *boot, u_int32_t chunk, u_char *b
 
     if (lseek(fd, offset, SEEK_SET) != offset)
     {
-        perror("Unable to seek FAT");
+        perr("Unable to seek FAT");
         return 1;
     }
     
     if (deblock_read(fd, buffer, length) != length)
     {
-        perror("Unable to read FAT");
+        perr("Unable to read FAT");
         return 1;
     }
 
@@ -278,13 +277,13 @@ static int _writechunks(int fd, struct bootblock *boot, u_int32_t chunk, u_char
         
         if (lseek(fd, offset, SEEK_SET) != offset)
         {
-            perror("Unable to seek FAT");
+            perr("Unable to seek FAT");
             return 1;
         }
         
         if (deblock_write(fd, buffer, length) != length)
         {
-            perror("Unable to write FAT");
+            perr("Unable to write FAT");
             return 1;
         }
     }
@@ -314,7 +313,7 @@ readfat(fs, boot, no, fp)
         buffer = malloc(FAT_CHUNK_SIZE);
         if (buffer == NULL)
         {
-            perror("No space for FAT");
+            perr("No space for FAT");
             return FSFATAL;
         }
         
@@ -328,7 +327,7 @@ readfat(fs, boot, no, fp)
         
 	fat = calloc(boot->NumClusters, sizeof(struct fatEntry));
 	if (fat == NULL) {
-		perror("No space for FAT");
+		perr("No space for FAT");
 		free(buffer);
 		return FSFATAL;
 	}
@@ -690,7 +689,7 @@ writefat(fs, boot, fat, correct_fat)
         /* Allocate a buffer for the current FAT chunk */
 	buffer = malloc(FAT_CHUNK_SIZE);
 	if (buffer == NULL) {
-		perror("No space for FAT");
+		perr("No space for FAT");
 		return FSFATAL;
 	}
 
