@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -96,6 +95,7 @@ mount_ufs(argc, argv)
 {
 	extern int optreset;
 	struct ufs_args args;
+	struct statfs fsinfo;
 	int ch, mntflags, noasync;
 	char *fs_name;
 
@@ -124,12 +124,16 @@ mount_ufs(argc, argv)
         args.fspec = argv[0];		/* The name of the device file. */
 	fs_name = argv[1];		/* The mount point. */
 
-#define DEFAULT_ROOTUID	-2
-	args.export.ex_root = DEFAULT_ROOTUID;
-	if (mntflags & MNT_RDONLY)
-		args.export.ex_flags = MNT_EXRDONLY;
-	else
-		args.export.ex_flags = 0;
+	/*
+	 * In the case of a mount point update, the mount system call below
+	 * will used, and it will succeed even if the volume is not UFS. Thus,
+	 * the noasync flag should be forced unless the volume actually is UFS.
+	 */
+	if (statfs(fs_name, &fsinfo) == 0)
+		if (strncmp(fsinfo.f_mntonname, fs_name, MFSNAMELEN) == 0)
+			if (strncmp(fsinfo.f_fstypename, "ufs", MFSNAMELEN)
+			    != 0)
+				noasync = 1;
 
 	/* default to async by setting the flag unless noasync was specified */
 	if (mount("ufs", fs_name, (mntflags | (noasync ? 0 : MNT_ASYNC)), &args)
