@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -79,6 +78,8 @@
 #include <ufs/ufs/dinode.h>
 #include <ufs/ufs/dir.h>
 #include <ufs/ffs/fs.h>
+#include <string.h>
+#include <stdlib.h>
 
 #ifdef linux
 #define MAXPHYSIO (64 * 1024)
@@ -124,7 +125,6 @@
 /*
  * variables set up by front end.
  */
-extern int	mfs;		/* run as the memory based filesystem */
 extern int	Nflag;		/* run mkfs without writing file system */
 extern int	Oflag;		/* format as an 4.3BSD file system */
 extern int	fssize;		/* file system size */
@@ -151,10 +151,10 @@ extern int	maxbpg;		/* maximum blocks per file in a cyl group */
 extern int	nrpos;		/* # of distinguished rotational positions */
 extern int	bbsize;		/* boot block size */
 extern int	sbsize;		/* superblock size */
+extern int	avgfilesize;	/* expected average file size */
+extern int	avgfilesperdir;	/* expected number of files per directory */
 extern u_long	memleft;	/* virtual memory available */
 extern caddr_t	membase;	/* start address of memory based filesystem */
-extern caddr_t	malloc(u_long), calloc(u_long, u_long);
-caddr_t realloc(char *, u_long);
 
 union {
 	struct fs fs;
@@ -200,16 +200,20 @@ daddr_t alloc(int size, int mode);
 long calcipg(long cpg, long bpcg, off_t *usedbp);
 void iput(register struct dinode *ip, register ino_t ino);
 void started();
-caddr_t malloc(register u_long size);
-caddr_t realloc(char *ptr, u_long size);
-char *calloc(u_long size, u_long numelm);
-void free(char *ptr);
 void rdfs(daddr_t bno, int size, char *bf);
 void wtfs(daddr_t bno, int size, char *bf);
 int isblock(struct fs *fs, unsigned char *cp, int h);
 void clrblock(struct fs *fs, unsigned char *cp, int h);
 void setblock(struct fs *fs, unsigned char *cp, int h);
 
+#ifndef __APPLE__
+/* Ifndef __APPLE__, replace libc function with one suited to our needs. */
+caddr_t malloc(register u_long size);
+caddr_t realloc(char *ptr, u_long size);
+char *calloc(u_long size, u_long numelm);
+void free(char *ptr);
+#endif /* __APPLE__ */
+
 void
 mkfs(fsys, fi, fo)
 	char *fsys;
@@ -221,7 +225,6 @@ mkfs(fsys, fi, fo)
 	off_t usedb;
 	long mapcramped, inodecramped;
 	long postblsize, rotblsize, totalsbsize;
-	int ppid, status;
 	time_t utime;
 	quad_t sizepb;
 	void started();
@@ -229,25 +232,6 @@ mkfs(fsys, fi, fo)
 #ifndef STANDALONE
 	time(&utime);
 #endif
-	if (mfs) {
-		ppid = getpid();
-		(void) signal(SIGUSR1, started);
-		if (i = fork()) {
-			if (i == -1) {
-				perror("mfs");
-				exit(10);
-			}
-			if (waitpid(i, &status, 0) != -1 && WIFEXITED(status))
-				exit(WEXITSTATUS(status));
-			exit(11);
-			/* NOTREACHED */
-		}
-		(void)malloc(0);
-		if (fssize * sectorsize > memleft)
-			fssize = (memleft - 16384) / sectorsize;
-		if ((membase = malloc(fssize * sectorsize)) == 0)
-			exit(12);
-	}
 	fsi = fi;
 	fso = fo;
 	if (Oflag) {
@@ -284,6 +268,17 @@ mkfs(fsys, fi, fo)
 		printf("preposterous ntrak %d\n", sblock.fs_ntrak), exit(14);
 	if (sblock.fs_nsect <= 0)
 		printf("preposterous nsect %d\n", sblock.fs_nsect), exit(15);
+	/*
+	 * collect and verify the filesystem density info
+	 */
+	sblock.fs_avgfilesize = avgfilesize;
+	sblock.fs_avgfpdir = avgfilesperdir;
+	if (sblock.fs_avgfilesize <= 0)
+		printf("illegal expected average file size %d\n",
+		    sblock.fs_avgfilesize), exit(14);
+	if (sblock.fs_avgfpdir <= 0)
+		printf("illegal expected number of files per directory %d\n",
+		    sblock.fs_avgfpdir), exit(15);
 	/*
 	 * collect and verify the block and fragment sizes
 	 */
@@ -659,7 +654,7 @@ next:
 		    NSPF(&sblock);
 		warn = 0;
 	}
-	if (warn && !mfs) {
+	if (warn) {
 		printf("Warning: %d sector(s) in last cylinder unallocated\n",
 		    sblock.fs_spc -
 		    (fssize * NSPF(&sblock) - (sblock.fs_ncyl - 1)
@@ -697,35 +692,29 @@ next:
 	/*
 	 * Dump out summary information about file system.
 	 */
-	if (!mfs) {
-		printf("%s:\t%d sectors in %d %s of %d tracks, %d sectors\n",
+	printf("%s:\t%d sectors in %d %s of %d tracks, %d sectors\n",
 		    fsys, sblock.fs_size * NSPF(&sblock), sblock.fs_ncyl,
 		    "cylinders", sblock.fs_ntrak, sblock.fs_nsect);
 #define B2MBFACTOR (1 / (1024.0 * 1024.0))
-		printf("\t%.1fMB in %d cyl groups (%d c/g, %.2fMB/g, %d i/g)\n",
+	printf("\t%.1fMB in %d cyl groups (%d c/g, %.2fMB/g, %d i/g)\n",
 		    (float)sblock.fs_size * sblock.fs_fsize * B2MBFACTOR,
 		    sblock.fs_ncg, sblock.fs_cpg,
 		    (float)sblock.fs_fpg * sblock.fs_fsize * B2MBFACTOR,
 		    sblock.fs_ipg);
 #undef B2MBFACTOR
-	}
 	/*
 	 * Now build the cylinders group blocks and
 	 * then print out indices of cylinder groups.
 	 */
-	if (!mfs)
-		printf("super-block backups (for fsck -b #) at:");
+	printf("super-block backups (for fsck -b #) at:");
 	for (cylno = 0; cylno < sblock.fs_ncg; cylno++) {
 		initcg(cylno, utime);
-		if (mfs)
-			continue;
 		if (cylno % 8 == 0)
 			printf("\n");
 		printf(" %d,", fsbtodb(&sblock, cgsblock(&sblock, cylno)));
 	}
-	if (!mfs)
-		printf("\n");
-	if (Nflag && !mfs)
+	printf("\n");
+	if (Nflag)
 		exit(0);
 	/*
 	 * Now construct the initial file system,
@@ -769,18 +758,6 @@ next:
 		wtfs(fsbtodb(&sblock, cgsblock(&sblock, cylno)),
 		    sbsize, (char *)&sblock);
 #endif  /* BIG_ENDIAN_INTEL_FS */
-	/*
-	 * Notify parent process of success.
-	 * Dissociate from session and tty.
-	 */
-	if (mfs) {
-		kill(ppid, SIGUSR1);
-		(void) setsid();
-		(void) close(0);
-		(void) close(1);
-		(void) close(2);
-		(void) chdir("/");
-	}
 }
 
 /*
@@ -794,7 +771,7 @@ initcg(cylno, utime)
 	daddr_t cbase, d, dlower, dupper, dmax, blkno;
 	long i;
 #ifdef __APPLE__
-	daddr_t wtfsbno;
+	daddr_t wtfsbno = 0;
 	char *	bufferPtr = NULL;
 	char * 	alignedBufPtr;
 	int	pageSize;
@@ -1062,6 +1039,8 @@ fsinit(utime)
 			    DIRSIZ(0, &lost_found_dir[2]));
 	}
 	node.di_mode = IFDIR | UMASK;
+	node.di_uid = geteuid();
+	node.di_gid = getegid();
 	node.di_nlink = 2;
 	node.di_size = sblock.fs_bsize;
 	node.di_db[0] = alloc(node.di_size, node.di_mode);
@@ -1083,10 +1062,9 @@ fsinit(utime)
 	/*
 	 * create the root directory
 	 */
-	if (mfs)
-		node.di_mode = IFDIR | 01777;
-	else
-		node.di_mode = IFDIR | UMASK;
+	node.di_mode = IFDIR | UMASK;
+	node.di_uid = geteuid();
+	node.di_gid = getegid();
 	node.di_nlink = PREDEFDIR;
 	if (Oflag)
 		node.di_size = makedir((struct direct *)oroot_dir, PREDEFDIR);
@@ -1376,11 +1354,6 @@ rdfs(bno, size, bf)
 	off64_t temp64;
 	int n;
 
-	if (mfs) {
-		memmove(bf, membase + bno * sectorsize, size);
-		return;
-	}
-
 	temp64 = bno;
 	temp64 *= sectorsize;
 	if (dklseek(fsi, temp64, 0) < 0) {
@@ -1408,10 +1381,6 @@ wtfs(bno, size, bf)
 	off64_t temp64;
 	int n;
 
-	if (mfs) {
-		memmove(membase + bno * sectorsize, bf, size);
-		return;
-	}
 	if (Nflag)
 		return;
 
