@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * "Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.0 (the 'License').  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
  * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License."
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -65,6 +64,8 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
+#include <mach/boolean.h>
+#include <sys/vm.h>
 
 #ifdef linux
 #include <fcntl.h>
@@ -99,6 +100,7 @@
 #include <string.h>
 #include <syslog.h>
 #include <unistd.h>
+#include <err.h>
 
 #ifdef __APPLE__
 int dkdisklabel __P((int fd, struct disklabel * lp));
@@ -229,6 +231,7 @@ int	unlabeled;
 char	device[MAXPATHLEN];
 char	*progname;
 
+extern void mkfs(char *, int, int);
 void usage();
 
 int
@@ -246,7 +249,7 @@ main(argc, argv)
 	struct stat st;
 	struct statfs *mp;
 	int fsi, fso, len, n;
-	char *cp, *s1, *s2, *special, *opstring, buf[BUFSIZ];
+	char *cp, *s1, *s2, *special, *opstring;
 #ifdef __APPLE__
 	char *	filesystem_name = "untitled"; /* filesystem name */
 #endif __APPLE__
@@ -361,9 +364,6 @@ main(argc, argv)
 			break;
 #ifdef __APPLE__
 		case 'v':
-		    	if (strchr(optarg, ':') || strchr(optarg, '/'))
-			    	fatal("%s: volume name contains invalid "
-				      "characters ':' or '/'", optarg);
 		    	filesystem_name = optarg;
 			break;
 #endif __APPLE__
