@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -34,6 +35,7 @@
 #include <sys/errno.h>
 #include <sys/stat.h>
 #include <sys/sysctl.h>
+#include <sys/vmmeter.h>
 
 #include <err.h>
 #include <errno.h>
@@ -44,6 +46,10 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <openssl/sha.h>
+
+#include <architecture/byte_order.h>
+
 #include <CoreFoundation/CFString.h>
 #include <CoreFoundation/CFStringEncodingExt.h>
 
@@ -71,8 +77,8 @@ typedef HFSMasterDirectoryBlock HFS_MDB;
 struct filefork {
 	UInt16	startBlock;
 	UInt16	blockCount;
-	SInt32	logicalSize;
-	SInt32	physicalSize;
+	UInt32	logicalSize;
+	UInt32	physicalSize;
 };
 
 struct filefork	gDTDBFork, gSystemFork, gReadMeFork;
@@ -133,8 +139,17 @@ static UInt32 DivideAndRoundUp __P((UInt32 numerator, UInt32 denominator));
 static int ConvertUTF8toUnicode __P((const UInt8* source, UInt32 bufsize,
 		UniChar* unibuf, UInt16 *charcount));
 
-static int getencodinghint(char *name);
+static int getencodinghint(unsigned char *name);
 
+#define VOLUMEUUIDVALUESIZE 2
+typedef union VolumeUUID {
+	UInt32 value[VOLUMEUUIDVALUESIZE];
+	struct {
+		UInt32 high;
+		UInt32 low;
+	} v;
+} VolumeUUID;
+void GenerateVolumeUUID(VolumeUUID *newVolumeID);
 
 void SETOFFSET (void *buffer, UInt16 btNodeSize, SInt16 recOffset, SInt16 vecOffset);
 #define SETOFFSET(buf,ndsiz,offset,rec)		\
@@ -237,12 +252,10 @@ make_hfs(const DriveInfo *driveInfo,
 		gSystemFork.blockCount = BYTESTOBLKS(gSystemFork.logicalSize, mdbp->drAlBlkSiz);
 		gSystemFork.physicalSize = gSystemFork.blockCount * mdbp->drAlBlkSiz;
 
-		(UInt16)mdbp->drFreeBks -= gDTDBFork.blockCount +
-				   	gReadMeFork.blockCount +
-				   	gSystemFork.blockCount;
+		mdbp->drFreeBks -= gDTDBFork.blockCount + gReadMeFork.blockCount + gSystemFork.blockCount;
 		mdbp->drEmbedExtent.startBlock = mdbp->drNmAlBlks - (UInt16)mdbp->drFreeBks;
 		mdbp->drEmbedExtent.blockCount = (UInt16)mdbp->drFreeBks;
-		(UInt16)mdbp->drFreeBks = 0;
+		mdbp->drFreeBks = 0;
 	}
 
 
@@ -312,7 +325,7 @@ make_hfsplus(const DriveInfo *driveInfo, hfsparams_t *defaults)
 	HFSPlusVolumeHeader	*header = NULL;
 	UInt32			temp;
 	UInt32			bits;
-	UInt32 bytesUsed;
+	UInt32 			bytesUsed;
 
 	/* --- Create an HFS Plus header:  */
 
@@ -421,8 +434,7 @@ make_hfsplus(const DriveInfo *driveInfo, hfsparams_t *defaults)
 	    sector = header->journalInfoBlock * sectorsPerBlock;
 	    WriteJournalInfo(driveInfo, sector, defaults, header, nodeBuffer);
 	}
-
-
+	
 	/*--- WRITE VOLUME HEADER TO DISK:  */
 
 	/* write header last in case we fail along the way */
@@ -475,6 +487,8 @@ InitMDB(hfsparams_t *defaults, UInt32 driveBlocks, HFS_MDB *mdbp)
 	UInt32	timeStamp;
 	UInt16	bitmapBlocks;
 	UInt32	alignment;
+	VolumeUUID	newVolumeUUID;	
+	VolumeUUID*	finderInfoUUIDPtr;
 
 	alignment = defaults->hfsAlignment;
 	bzero(mdbp, kBytesPerSector);
@@ -509,7 +523,7 @@ InitMDB(hfsparams_t *defaults, UInt32 driveBlocks, HFS_MDB *mdbp)
 	mdbp->drAlBlkSiz = alBlkSize;
 	mdbp->drClpSiz = defaults->dataClumpSize;
 	mdbp->drNxtCNID = defaults->nextFreeFileID;
-	(UInt16)mdbp->drFreeBks = numAlBlks;
+	mdbp->drFreeBks = numAlBlks;
 	
 	/*
 	 * Map UTF-8 input into a Mac encoding.
@@ -521,7 +535,7 @@ InitMDB(hfsparams_t *defaults, UInt32 driveBlocks, HFS_MDB *mdbp)
 		CFIndex maxchars;
 		Boolean cfOK;
 	
-		cfstr = CFStringCreateWithCString(kCFAllocatorDefault, defaults->volumeName, kCFStringEncodingUTF8);
+		cfstr = CFStringCreateWithCString(kCFAllocatorDefault, (char *)defaults->volumeName, kCFStringEncodingUTF8);
 
 		/* Find out what Mac encoding to use: */
 		maxchars = MIN(sizeof(unibuf)/sizeof(UniChar), CFStringGetLength(cfstr));
@@ -548,13 +562,13 @@ InitMDB(hfsparams_t *defaults, UInt32 driveBlocks, HFS_MDB *mdbp)
 	mdbp->drXTFlSize = mdbp->drXTClpSiz = defaults->extentsClumpSize;
 	mdbp->drXTExtRec[0].startBlock = 0;
 	mdbp->drXTExtRec[0].blockCount = mdbp->drXTFlSize / alBlkSize;
-	(UInt16)mdbp->drFreeBks -= mdbp->drXTExtRec[0].blockCount;
+	mdbp->drFreeBks -= mdbp->drXTExtRec[0].blockCount;
 
 	mdbp->drCTFlSize = mdbp->drCTClpSiz = defaults->catalogClumpSize;
 	mdbp->drCTExtRec[0].startBlock = mdbp->drXTExtRec[0].startBlock +
 					 mdbp->drXTExtRec[0].blockCount;
 	mdbp->drCTExtRec[0].blockCount = mdbp->drCTFlSize / alBlkSize;
-	(UInt16)mdbp->drFreeBks -= mdbp->drCTExtRec[0].blockCount;
+	mdbp->drFreeBks -= mdbp->drCTExtRec[0].blockCount;
 
 	if (defaults->flags & kMakeHFSWrapper) {
 		mdbp->drFilCnt = mdbp->drNmFls = kWapperFileCount;
@@ -569,6 +583,11 @@ InitMDB(hfsparams_t *defaults, UInt32 driveBlocks, HFS_MDB *mdbp)
 		mdbp->drAtrb |= kHFSVolumeSparedBlocksMask;
 		mdbp->drAtrb |= kHFSVolumeSoftwareLockMask;
 	}
+	/* Generate and write UUID for the HFS disk */
+	GenerateVolumeUUID(&newVolumeUUID);
+	finderInfoUUIDPtr = (VolumeUUID *)(&mdbp->drFndrInfo[6]);
+	finderInfoUUIDPtr->v.high = NXSwapHostLongToBig(newVolumeUUID.v.high); 
+	finderInfoUUIDPtr->v.low = NXSwapHostLongToBig(newVolumeUUID.v.low); 
 }
 
 
@@ -605,6 +624,8 @@ InitVH(hfsparams_t *defaults, UInt64 sectors, HFSPlusVolumeHeader *hp)
 	UInt16	burnedBlocksBeforeVH = 0;
 	UInt16	burnedBlocksAfterAltVH = 0;
 	UInt32  nextBlock;
+	VolumeUUID	newVolumeUUID;	
+	VolumeUUID*	finderInfoUUIDPtr;
 	
 	bzero(hp, kBytesPerSector);
 
@@ -708,6 +729,12 @@ InitVH(hfsparams_t *defaults, UInt64 sectors, HFSPlusVolumeHeader *hp)
 	 */
 	hp->nextAllocation = blocksUsed - 1 - burnedBlocksAfterAltVH +
 		10 * (hp->catalogFile.clumpSize / hp->blockSize);
+	
+	/* Generate and write UUID for the HFS+ disk */
+	GenerateVolumeUUID(&newVolumeUUID);
+	finderInfoUUIDPtr = (VolumeUUID *)(&hp->finderInfo[24]);
+	finderInfoUUIDPtr->v.high = NXSwapHostLongToBig(newVolumeUUID.v.high); 
+	finderInfoUUIDPtr->v.low = NXSwapHostLongToBig(newVolumeUUID.v.low); 
 }
 
 
@@ -1098,14 +1125,14 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 	ckp = (HFSPlusCatalogKey *)((UInt8 *)buffer + offset);
 	
 	/* Use CFString functions to get a HFSPlus Canonical name */
-	cfstr = CFStringCreateWithCString(kCFAllocatorDefault, dp->volumeName, kCFStringEncodingUTF8);
+	cfstr = CFStringCreateWithCString(kCFAllocatorDefault, (char *)dp->volumeName, kCFStringEncodingUTF8);
 	cfOK = _CFStringGetFileSystemRepresentation(cfstr, canonicalName, sizeof(canonicalName));
 
 	if (!cfOK || ConvertUTF8toUnicode(canonicalName, sizeof(ckp->nodeName.unicode),
 		ckp->nodeName.unicode, &ckp->nodeName.length)) {
 
 		/* On conversion errors "untitled" is used as a fallback. */
-		(void) ConvertUTF8toUnicode(kDefaultVolumeNameStr,
+		(void) ConvertUTF8toUnicode((UInt8 *)kDefaultVolumeNameStr,
 									sizeof(ckp->nodeName.unicode),
 									ckp->nodeName.unicode,
 									&ckp->nodeName.length);
@@ -1164,7 +1191,7 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 
 		/* File record #1 */
 		ckp = (HFSPlusCatalogKey *)((UInt8 *)buffer + offset);
-		(void) ConvertUTF8toUnicode(HFS_JOURNAL_FILE, sizeof(ckp->nodeName.unicode),
+		(void) ConvertUTF8toUnicode((UInt8 *)HFS_JOURNAL_FILE, sizeof(ckp->nodeName.unicode),
 		                            ckp->nodeName.unicode, &ckp->nodeName.length);
 		ckp->nodeName.length = SWAP_BE16 (ckp->nodeName.length);
 		uBytes1 = sizeof(UniChar) * SWAP_BE16 (ckp->nodeName.length);
@@ -1197,7 +1224,7 @@ InitCatalogRoot_HFSPlus(const hfsparams_t *dp, const HFSPlusVolumeHeader *header
 
 		/* File record #2 */
 		ckp = (HFSPlusCatalogKey *)((UInt8 *)buffer + offset);
-		(void) ConvertUTF8toUnicode(HFS_JOURNAL_INFO, sizeof(ckp->nodeName.unicode),
+		(void) ConvertUTF8toUnicode((UInt8 *)HFS_JOURNAL_INFO, sizeof(ckp->nodeName.unicode),
 		                            ckp->nodeName.unicode, &ckp->nodeName.length);
 		ckp->nodeName.length = SWAP_BE16 (ckp->nodeName.length);
 		uBytes2 = sizeof(UniChar) * SWAP_BE16 (ckp->nodeName.length);
@@ -1293,7 +1320,7 @@ InitFirstCatalogLeaf(const hfsparams_t *dp, void * buffer, int wrapper)
 	 * First record is always the root directory...
 	 */
 	ckp = (HFSCatalogKey *)((UInt8 *)buffer + offset);
-	ckp->nodeName[0]	= strlen(dp->volumeName);
+	ckp->nodeName[0]	= strlen((char *)dp->volumeName);
 	bcopy(dp->volumeName, &ckp->nodeName[1], ckp->nodeName[0]);
 	ckp->keyLength		= 1 + 4 + ((ckp->nodeName[0] + 2) & 0xFE);  /* pad to word */
 	ckp->parentID		= SWAP_BE32 (kHFSRootParentID);
@@ -1515,7 +1542,7 @@ InitCatalogRoot_HFS(const hfsparams_t *dp, void * buffer)
 	ckp = (HFSCatalogKey *)((UInt8 *)buffer + offset);
 	ckp->keyLength		= kHFSCatalogKeyMaximumLength;
 	ckp->parentID		= SWAP_BE32 (kHFSRootParentID);
-	ckp->nodeName[0]	= strlen(dp->volumeName);
+	ckp->nodeName[0]	= strlen((char *)dp->volumeName);
 	bcopy(dp->volumeName, &ckp->nodeName[1], ckp->nodeName[0]);
 	offset += ckp->keyLength + 1;
 
@@ -1978,7 +2005,7 @@ ConvertUTF8toUnicode(const UInt8* source, UInt32 bufsize, UniChar* unibuf,
  * Derive the encoding hint for the given name.
  */
 static int
-getencodinghint(char *name)
+getencodinghint(unsigned char *name)
 {
         int mib[3];
         size_t buflen = sizeof(int);
@@ -1992,7 +2019,7 @@ getencodinghint(char *name)
         mib[1] = vfc.vfc_typenum;
         mib[2] = HFS_ENCODINGHINT;
  
-	if (sysctl(mib, 3, &hint, &buflen, name, strlen(name) + 1) < 0)
+	if (sysctl(mib, 3, &hint, &buflen, name, strlen((char *)name) + 1) < 0)
  		goto error;
 	return (hint);
 error:
@@ -2000,3 +2027,88 @@ error:
 	return (hint);
 }
 
+
+/* Generate Volume UUID - similar to code existing in hfs_util */
+void GenerateVolumeUUID(VolumeUUID *newVolumeID) {
+	SHA_CTX context;
+	char randomInputBuffer[26];
+	unsigned char digest[20];
+	time_t now;
+	clock_t uptime;
+	int mib[2];
+	int sysdata;
+	char sysctlstring[128];
+	size_t datalen;
+	double sysloadavg[3];
+	struct vmtotal sysvmtotal;
+	
+	do {
+		/* Initialize the SHA-1 context for processing: */
+		SHA1_Init(&context);
+		
+		/* Now process successive bits of "random" input to seed the process: */
+		
+		/* The current system's uptime: */
+		uptime = clock();
+		SHA1_Update(&context, &uptime, sizeof(uptime));
+		
+		/* The kernel's boot time: */
+		mib[0] = CTL_KERN;
+		mib[1] = KERN_BOOTTIME;
+		datalen = sizeof(sysdata);
+		sysctl(mib, 2, &sysdata, &datalen, NULL, 0);
+		SHA1_Update(&context, &sysdata, datalen);
+		
+		/* The system's host id: */
+		mib[0] = CTL_KERN;
+		mib[1] = KERN_HOSTID;
+		datalen = sizeof(sysdata);
+		sysctl(mib, 2, &sysdata, &datalen, NULL, 0);
+		SHA1_Update(&context, &sysdata, datalen);
+
+		/* The system's host name: */
+		mib[0] = CTL_KERN;
+		mib[1] = KERN_HOSTNAME;
+		datalen = sizeof(sysctlstring);
+		sysctl(mib, 2, sysctlstring, &datalen, NULL, 0);
+		SHA1_Update(&context, sysctlstring, datalen);
+
+		/* The running kernel's OS release string: */
+		mib[0] = CTL_KERN;
+		mib[1] = KERN_OSRELEASE;
+		datalen = sizeof(sysctlstring);
+		sysctl(mib, 2, sysctlstring, &datalen, NULL, 0);
+		SHA1_Update(&context, sysctlstring, datalen);
+
+		/* The running kernel's version string: */
+		mib[0] = CTL_KERN;
+		mib[1] = KERN_VERSION;
+		datalen = sizeof(sysctlstring);
+		sysctl(mib, 2, sysctlstring, &datalen, NULL, 0);
+		SHA1_Update(&context, sysctlstring, datalen);
+
+		/* The system's load average: */
+		datalen = sizeof(sysloadavg);
+		getloadavg(sysloadavg, 3);
+		SHA1_Update(&context, &sysloadavg, datalen);
+
+		/* The system's VM statistics: */
+		mib[0] = CTL_VM;
+		mib[1] = VM_METER;
+		datalen = sizeof(sysvmtotal);
+		sysctl(mib, 2, &sysvmtotal, &datalen, NULL, 0);
+		SHA1_Update(&context, &sysvmtotal, datalen);
+
+		/* The current GMT (26 ASCII characters): */
+		time(&now);
+		strncpy(randomInputBuffer, asctime(gmtime(&now)), 26);	/* "Mon Mar 27 13:46:26 2000" */
+		SHA1_Update(&context, randomInputBuffer, 26);
+		
+		/* Pad the accumulated input and extract the final digest hash: */
+		SHA1_Final(digest, &context);
+	
+		memcpy(newVolumeID, digest, sizeof(*newVolumeID));
+	} while ((newVolumeID->v.high == 0) || (newVolumeID->v.low == 0));
+}
+
+
