@@ -3,19 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -141,7 +142,7 @@ main(argc, argv)
 	struct statfs *mp;
 	int n;
 	
-	if (progname = strrchr(*argv, '/'))
+	if ((progname = strrchr(*argv, '/')))
 		++progname;
 	else
 		progname = *argv;
@@ -185,6 +186,8 @@ main(argc, argv)
 			gBlockSize = atoi(optarg);
 			if (gBlockSize < HFSMINBSIZE)
 				fatal("%s: bad allocation block size (too small)", optarg);
+			if (gBlockSize > MAXBSIZE) 
+				fatal("%s: bad allocation block size (too large)", optarg);
 			break;
 
 		case 'c':
@@ -296,7 +299,7 @@ static void getnodeopts(char* optlist)
 	char *p;
 	UInt32 ndsize;
 	
-	while((ndarg = strsep(&strp, ",")) != NULL && *ndarg != NULL) {
+	while((ndarg = strsep(&strp, ",")) != NULL && *ndarg != '\0') {
 
 		p = strchr(ndarg, '=');
 		if (p == NULL)
@@ -306,20 +309,20 @@ static void getnodeopts(char* optlist)
 
 		switch (*ndarg) {
 		case 'c':
-			if (ndsize < 4096 || ndsize > 32768 || (ndsize & ndsize-1) != 0)
+			if (ndsize < 4096 || ndsize > 32768 || (ndsize & (ndsize-1)) != 0)
 				fatal("%s: invalid catalog b-tree node size", ndarg);
 			catnodesiz = ndsize;
 			gUserCatNodeSize = TRUE;
 			break;
 
 		case 'e':
-			if (ndsize < 1024 || ndsize > 32768 || (ndsize & ndsize-1) != 0)
+			if (ndsize < 1024 || ndsize > 32768 || (ndsize & (ndsize-1)) != 0)
 				fatal("%s: invalid extents b-tree node size", ndarg);
 			extnodesiz = ndsize;
 			break;
 
 		case 'a':
-			if (ndsize < 1024 || ndsize > 32768 || (ndsize & ndsize-1) != 0)
+			if (ndsize < 1024 || ndsize > 32768 || (ndsize & (ndsize-1)) != 0)
 				fatal("%s: invalid atrribute b-tree node size", ndarg);
 			atrnodesiz = ndsize;
 			break;
@@ -338,7 +341,7 @@ static void getclumpopts(char* optlist)
 	char *p;
 	UInt32 clpblocks;
 	
-	while((ndarg = strsep(&strp, ",")) != NULL && *ndarg != NULL) {
+	while((ndarg = strsep(&strp, ",")) != NULL && *ndarg != '\0') {
 
 		p = strchr(ndarg, '=');
 		if (p == NULL)
@@ -449,7 +452,7 @@ static void validate_hfsplus_block_size(UInt64 sectorCount, UInt32 sectorSize)
 		}
 	} else {
 		/* Make sure a user-specified block size is reasonable */
-		if ((gBlockSize & gBlockSize-1) != 0)
+		if ((gBlockSize & (gBlockSize-1)) != 0)
 			fatal("%s: bad HFS Plus allocation block size (must be a power of two)", optarg);
 	
 		if ((sectorCount / (gBlockSize / sectorSize)) > 0xFFFFFFFF)
@@ -565,7 +568,7 @@ hfs_newfs(char *device, int forceHFS, int isRaw)
 		 * this large should be 4K aligned for performance.
 		 */
 		if ((dip.totalSectors >= 0x40000000) && (dip.totalSectors & 7))
-			fatal("%s: partition size must be a multiple of 4K", device);
+		    fatal("%s: partition size not a multiple of 4K.", device);
 
 		hfsplus_params(dip.totalSectors, dip.sectorSize, &defaults);
 		if (gNoCreate == 0) {
@@ -650,7 +653,7 @@ static void hfsplus_params (UInt64 sectorCount, UInt32 sectorSize, hfsparams_t *
 	    defaults->journalSize = 512 * 1024 * 1024;
 	}
 
-	strncpy(defaults->volumeName, gVolumeName, sizeof(defaults->volumeName) - 1);
+	strncpy((char *)defaults->volumeName, gVolumeName, sizeof(defaults->volumeName) - 1);
 	defaults->volumeName[sizeof(defaults->volumeName) - 1] = '\0';
 
 	if (rsrclumpblks == 0) {
@@ -791,7 +794,7 @@ static void hfs_params(UInt32 sectorCount, UInt32 sectorSize, hfsparams_t *defau
 	defaults->attributesNodeSize = 0;
 	defaults->attributesClumpSize = 0;
 
-	strncpy(defaults->volumeName, gVolumeName, sizeof(defaults->volumeName) - 1);
+	strncpy((char *)defaults->volumeName, gVolumeName, sizeof(defaults->volumeName) - 1);
 	defaults->volumeName[sizeof(defaults->volumeName) - 1] = '\0';
 
 	/* Compute the default allocation block size */
