@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -76,7 +74,7 @@ chkquota_hfs(fsname, mntpt, qnp)
 	char *fsname, *mntpt;
 	register struct quotaname *qnp;
 {
-        int errs = 0;
+	int errs = 0;
 	
 	sync();
 
@@ -116,7 +114,7 @@ collectdata(const char* path, struct quotaname *qnp)
 	/* Search for items with uid != 0 */
 	query.qq_len = sizeof(struct quota_query);
 	query.qq_uid = 0;
-	options |= SRCHFS_NEGATEPARAMS;
+	options |= SRCHFS_NEGATEPARAMS | SRCHFS_SKIPLINKS;
 	searchblk.searchattrs.bitmapcount = ATTR_BIT_MAP_COUNT;
 	searchblk.searchattrs.commonattr = QQ_COMMON;
 	searchblk.searchparams1 = &query;
@@ -149,7 +147,7 @@ collectdata(const char* path, struct quotaname *qnp)
 
 		for (i = 0; i < nummatches; ++i) {
 			vntype = qap[i].qa_type;
-			filebytes = (vntype == VDIR) ? 0 :qap[i].qa_bytes;
+			filebytes = (vntype == VDIR) ? 0 : qap[i].qa_bytes;
 	
 			if (qnp->flags & HASGRP) {
 				fup = addid(qap[i].qa_gid, GRPQUOTA);
