@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -80,6 +78,12 @@ static char sccsid[] = "@(#)quotaon.c	8.1 (Berkeley) 6/6/93";
 #include <ufs/ufs/quota.h>
 #include <stdio.h>
 #include <fstab.h>
+#include <string.h>
+#include <stdlib.h>
+#include <unistd.h>
+
+/* Internal functions */
+void usage(char *whoami);
 
 char *qfname = QUOTAFILENAME;
 char *qfextension[] = INITQFNAMES;
@@ -89,7 +93,7 @@ int	gflag;		/* operate on group quotas */
 int	uflag;		/* operate on user quotas */
 int	vflag;		/* verbose */
 
-main(argc, argv)
+int main(argc, argv)
 	int argc;
 	char **argv;
 {
@@ -205,7 +209,7 @@ main(argc, argv)
 	exit(errs);
 }
 
-usage(whoami)
+void usage(whoami)
 	char *whoami;
 {
 
@@ -215,7 +219,7 @@ usage(whoami)
 }
 
 #ifdef __APPLE__
-quotaonoff(fst, offmode, type, qfpathname)
+int quotaonoff(fst, offmode, type, qfpathname)
 	register struct statfs *fst;
 	int offmode, type;
 	char *qfpathname;
@@ -244,7 +248,7 @@ quotaonoff(fst, offmode, type, qfpathname)
 	return (0);
 }
 #else
-quotaonoff(fs, offmode, type, qfpathname)
+int quotaonoff(fs, offmode, type, qfpathname)
 	register struct fstab *fs;
 	int offmode, type;
 	char *qfpathname;
@@ -277,7 +281,7 @@ quotaonoff(fs, offmode, type, qfpathname)
 /*
  * Check to see if target appears in list of size cnt.
  */
-oneof(target, list, cnt)
+int oneof(target, list, cnt)
 	register char *target, *list[];
 	int cnt;
 {
@@ -293,7 +297,7 @@ oneof(target, list, cnt)
  * Check to see if a particular quota is to be enabled.
  */
 #ifdef __APPLE__
-hasquota(fst, type, qfnamep)
+int hasquota(fst, type, qfnamep)
 	register struct statfs *fst;
 	int type;
 	char **qfnamep;
@@ -326,7 +330,7 @@ hasquota(fst, type, qfnamep)
 	return (1);
 }
 #else
-hasquota(fs, type, qfnamep)
+int hasquota(fs, type, qfnamep)
 	register struct fstab *fs;
 	int type;
 	char **qfnamep;
@@ -366,7 +370,7 @@ hasquota(fs, type, qfnamep)
 /*
  * Verify file system is mounted and not readonly.
  */
-readonly(fs)
+int readonly(fs)
 	register struct fstab *fs;
 {
 	struct statfs fsbuf;
